from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.profiling_time_period_truncation import ProfilingTimePeriodTruncation
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.check_search_filters import CheckSearchFilters
    from ..models.delete_stored_data_queue_job_parameters import (
        DeleteStoredDataQueueJobParameters,
    )
    from ..models.file_format_spec import FileFormatSpec
    from ..models.physical_table_name import PhysicalTableName
    from ..models.statistics_collector_search_filters import (
        StatisticsCollectorSearchFilters,
    )
    from ..models.table_current_data_quality_status_model import (
        TableCurrentDataQualityStatusModel,
    )
    from ..models.table_list_model_advanced_properties import (
        TableListModelAdvancedProperties,
    )
    from ..models.table_owner_spec import TableOwnerSpec


T = TypeVar("T", bound="TableListModel")


@_attrs_define
class TableListModel:
    """Table list model with a subset of parameters, excluding all nested objects.

    Attributes:
        connection_name (Union[Unset, str]): Connection name.
        table_hash (Union[Unset, int]): Table hash that identifies the table using a unique hash code.
        target (Union[Unset, PhysicalTableName]):
        labels (Union[Unset, List[str]]): List of labels applied to the table.
        disabled (Union[Unset, bool]): Disables all data quality checks on the table. Data quality checks will not be
            executed.
        stage (Union[Unset, str]): Stage name.
        filter_ (Union[Unset, str]): SQL WHERE clause added to the sensor queries.
        do_not_collect_error_samples_in_profiling (Union[Unset, bool]): Disable automatic collection of error samples in
            the profiling section. The profiling checks by default always collect error samples for failed data quality
            checks.
        always_collect_error_samples_in_monitoring (Union[Unset, bool]): Always collect error samples for failed
            monitoring checks. DQOps will not collect error samples automatically when the checks are executed by a
            scheduler or by running checks from the metadata tree. Error samples are always collected only when the checks
            are run from the check editor.
        priority (Union[Unset, int]): Table priority (1, 2, 3, 4, ...). The tables can be assigned a priority level. The
            table priority is copied into each data quality check result and a sensor result, enabling efficient grouping of
            more and less important tables during a data quality improvement project, when the data quality issues on higher
            priority tables are fixed before data quality issues on less important tables.
        owner (Union[Unset, TableOwnerSpec]):
        profiling_checks_result_truncation (Union[Unset, ProfilingTimePeriodTruncation]):
        file_format (Union[Unset, FileFormatSpec]):
        data_quality_status (Union[Unset, TableCurrentDataQualityStatusModel]): The table's most recent data quality
            status. It is a summary of the results of the most recently executed data quality checks on the table. Verify
            the value of the highest_severity_level to see if there are any data quality issues on the table. The values of
            severity levels are: 0 - all data quality checks passed, 1 - a warning was detected, 2 - an error was detected,
            3 - a fatal data quality issue was detected.
        has_any_configured_checks (Union[Unset, bool]): True when the table has any checks configured.
        has_any_configured_profiling_checks (Union[Unset, bool]): True when the table has any profiling checks
            configured.
        has_any_configured_monitoring_checks (Union[Unset, bool]): True when the table has any monitoring checks
            configured.
        has_any_configured_partition_checks (Union[Unset, bool]): True when the table has any partition checks
            configured.
        partitioning_configuration_missing (Union[Unset, bool]): True when the table has missing configuration of the
            "partition_by_column" column, making any partition checks fail when executed.
        run_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter, identifies which
            checks on which tables and columns should be executed.
        run_profiling_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter,
            identifies which checks on which tables and columns should be executed.
        run_monitoring_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter,
            identifies which checks on which tables and columns should be executed.
        run_partition_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter,
            identifies which checks on which tables and columns should be executed.
        collect_statistics_job_template (Union[Unset, StatisticsCollectorSearchFilters]):
        data_clean_job_template (Union[Unset, DeleteStoredDataQueueJobParameters]):
        advanced_properties (Union[Unset, TableListModelAdvancedProperties]): A dictionary of advanced properties that
            can be used for e.g. to support mapping data to data catalogs, a key/value dictionary.
        can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can update or delete this object.
        can_collect_statistics (Union[Unset, bool]): Boolean flag that decides if the current user can collect
            statistics.
        can_run_checks (Union[Unset, bool]): Boolean flag that decides if the current user can run checks.
        can_delete_data (Union[Unset, bool]): Boolean flag that decides if the current user can delete data (results).
        yaml_parsing_error (Union[Unset, str]): Optional parsing error that was captured when parsing the YAML file.
            This field is null when the YAML file is valid. If an error was captured, this field returns the file parsing
            error message and the file location.
    """

    connection_name: Union[Unset, str] = UNSET
    table_hash: Union[Unset, int] = UNSET
    target: Union[Unset, "PhysicalTableName"] = UNSET
    labels: Union[Unset, List[str]] = UNSET
    disabled: Union[Unset, bool] = UNSET
    stage: Union[Unset, str] = UNSET
    filter_: Union[Unset, str] = UNSET
    do_not_collect_error_samples_in_profiling: Union[Unset, bool] = UNSET
    always_collect_error_samples_in_monitoring: Union[Unset, bool] = UNSET
    priority: Union[Unset, int] = UNSET
    owner: Union[Unset, "TableOwnerSpec"] = UNSET
    profiling_checks_result_truncation: Union[Unset, ProfilingTimePeriodTruncation] = (
        UNSET
    )
    file_format: Union[Unset, "FileFormatSpec"] = UNSET
    data_quality_status: Union[Unset, "TableCurrentDataQualityStatusModel"] = UNSET
    has_any_configured_checks: Union[Unset, bool] = UNSET
    has_any_configured_profiling_checks: Union[Unset, bool] = UNSET
    has_any_configured_monitoring_checks: Union[Unset, bool] = UNSET
    has_any_configured_partition_checks: Union[Unset, bool] = UNSET
    partitioning_configuration_missing: Union[Unset, bool] = UNSET
    run_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    run_profiling_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    run_monitoring_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    run_partition_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    collect_statistics_job_template: Union[
        Unset, "StatisticsCollectorSearchFilters"
    ] = UNSET
    data_clean_job_template: Union[Unset, "DeleteStoredDataQueueJobParameters"] = UNSET
    advanced_properties: Union[Unset, "TableListModelAdvancedProperties"] = UNSET
    can_edit: Union[Unset, bool] = UNSET
    can_collect_statistics: Union[Unset, bool] = UNSET
    can_run_checks: Union[Unset, bool] = UNSET
    can_delete_data: Union[Unset, bool] = UNSET
    yaml_parsing_error: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connection_name = self.connection_name
        table_hash = self.table_hash
        target: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.target, Unset):
            target = self.target.to_dict()

        labels: Union[Unset, List[str]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels

        disabled = self.disabled
        stage = self.stage
        filter_ = self.filter_
        do_not_collect_error_samples_in_profiling = (
            self.do_not_collect_error_samples_in_profiling
        )
        always_collect_error_samples_in_monitoring = (
            self.always_collect_error_samples_in_monitoring
        )
        priority = self.priority
        owner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner, Unset):
            owner = self.owner.to_dict()

        profiling_checks_result_truncation: Union[Unset, str] = UNSET
        if not isinstance(self.profiling_checks_result_truncation, Unset):
            profiling_checks_result_truncation = (
                self.profiling_checks_result_truncation.value
            )

        file_format: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.file_format, Unset):
            file_format = self.file_format.to_dict()

        data_quality_status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data_quality_status, Unset):
            data_quality_status = self.data_quality_status.to_dict()

        has_any_configured_checks = self.has_any_configured_checks
        has_any_configured_profiling_checks = self.has_any_configured_profiling_checks
        has_any_configured_monitoring_checks = self.has_any_configured_monitoring_checks
        has_any_configured_partition_checks = self.has_any_configured_partition_checks
        partitioning_configuration_missing = self.partitioning_configuration_missing
        run_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_checks_job_template, Unset):
            run_checks_job_template = self.run_checks_job_template.to_dict()

        run_profiling_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_profiling_checks_job_template, Unset):
            run_profiling_checks_job_template = (
                self.run_profiling_checks_job_template.to_dict()
            )

        run_monitoring_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_monitoring_checks_job_template, Unset):
            run_monitoring_checks_job_template = (
                self.run_monitoring_checks_job_template.to_dict()
            )

        run_partition_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_partition_checks_job_template, Unset):
            run_partition_checks_job_template = (
                self.run_partition_checks_job_template.to_dict()
            )

        collect_statistics_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.collect_statistics_job_template, Unset):
            collect_statistics_job_template = (
                self.collect_statistics_job_template.to_dict()
            )

        data_clean_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data_clean_job_template, Unset):
            data_clean_job_template = self.data_clean_job_template.to_dict()

        advanced_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.advanced_properties, Unset):
            advanced_properties = self.advanced_properties.to_dict()

        can_edit = self.can_edit
        can_collect_statistics = self.can_collect_statistics
        can_run_checks = self.can_run_checks
        can_delete_data = self.can_delete_data
        yaml_parsing_error = self.yaml_parsing_error

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connection_name is not UNSET:
            field_dict["connection_name"] = connection_name
        if table_hash is not UNSET:
            field_dict["table_hash"] = table_hash
        if target is not UNSET:
            field_dict["target"] = target
        if labels is not UNSET:
            field_dict["labels"] = labels
        if disabled is not UNSET:
            field_dict["disabled"] = disabled
        if stage is not UNSET:
            field_dict["stage"] = stage
        if filter_ is not UNSET:
            field_dict["filter"] = filter_
        if do_not_collect_error_samples_in_profiling is not UNSET:
            field_dict["do_not_collect_error_samples_in_profiling"] = (
                do_not_collect_error_samples_in_profiling
            )
        if always_collect_error_samples_in_monitoring is not UNSET:
            field_dict["always_collect_error_samples_in_monitoring"] = (
                always_collect_error_samples_in_monitoring
            )
        if priority is not UNSET:
            field_dict["priority"] = priority
        if owner is not UNSET:
            field_dict["owner"] = owner
        if profiling_checks_result_truncation is not UNSET:
            field_dict["profiling_checks_result_truncation"] = (
                profiling_checks_result_truncation
            )
        if file_format is not UNSET:
            field_dict["file_format"] = file_format
        if data_quality_status is not UNSET:
            field_dict["data_quality_status"] = data_quality_status
        if has_any_configured_checks is not UNSET:
            field_dict["has_any_configured_checks"] = has_any_configured_checks
        if has_any_configured_profiling_checks is not UNSET:
            field_dict["has_any_configured_profiling_checks"] = (
                has_any_configured_profiling_checks
            )
        if has_any_configured_monitoring_checks is not UNSET:
            field_dict["has_any_configured_monitoring_checks"] = (
                has_any_configured_monitoring_checks
            )
        if has_any_configured_partition_checks is not UNSET:
            field_dict["has_any_configured_partition_checks"] = (
                has_any_configured_partition_checks
            )
        if partitioning_configuration_missing is not UNSET:
            field_dict["partitioning_configuration_missing"] = (
                partitioning_configuration_missing
            )
        if run_checks_job_template is not UNSET:
            field_dict["run_checks_job_template"] = run_checks_job_template
        if run_profiling_checks_job_template is not UNSET:
            field_dict["run_profiling_checks_job_template"] = (
                run_profiling_checks_job_template
            )
        if run_monitoring_checks_job_template is not UNSET:
            field_dict["run_monitoring_checks_job_template"] = (
                run_monitoring_checks_job_template
            )
        if run_partition_checks_job_template is not UNSET:
            field_dict["run_partition_checks_job_template"] = (
                run_partition_checks_job_template
            )
        if collect_statistics_job_template is not UNSET:
            field_dict["collect_statistics_job_template"] = (
                collect_statistics_job_template
            )
        if data_clean_job_template is not UNSET:
            field_dict["data_clean_job_template"] = data_clean_job_template
        if advanced_properties is not UNSET:
            field_dict["advanced_properties"] = advanced_properties
        if can_edit is not UNSET:
            field_dict["can_edit"] = can_edit
        if can_collect_statistics is not UNSET:
            field_dict["can_collect_statistics"] = can_collect_statistics
        if can_run_checks is not UNSET:
            field_dict["can_run_checks"] = can_run_checks
        if can_delete_data is not UNSET:
            field_dict["can_delete_data"] = can_delete_data
        if yaml_parsing_error is not UNSET:
            field_dict["yaml_parsing_error"] = yaml_parsing_error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.check_search_filters import CheckSearchFilters
        from ..models.delete_stored_data_queue_job_parameters import (
            DeleteStoredDataQueueJobParameters,
        )
        from ..models.file_format_spec import FileFormatSpec
        from ..models.physical_table_name import PhysicalTableName
        from ..models.statistics_collector_search_filters import (
            StatisticsCollectorSearchFilters,
        )
        from ..models.table_current_data_quality_status_model import (
            TableCurrentDataQualityStatusModel,
        )
        from ..models.table_list_model_advanced_properties import (
            TableListModelAdvancedProperties,
        )
        from ..models.table_owner_spec import TableOwnerSpec

        d = src_dict.copy()
        connection_name = d.pop("connection_name", UNSET)

        table_hash = d.pop("table_hash", UNSET)

        _target = d.pop("target", UNSET)
        target: Union[Unset, PhysicalTableName]
        if isinstance(_target, Unset):
            target = UNSET
        else:
            target = PhysicalTableName.from_dict(_target)

        labels = cast(List[str], d.pop("labels", UNSET))

        disabled = d.pop("disabled", UNSET)

        stage = d.pop("stage", UNSET)

        filter_ = d.pop("filter", UNSET)

        do_not_collect_error_samples_in_profiling = d.pop(
            "do_not_collect_error_samples_in_profiling", UNSET
        )

        always_collect_error_samples_in_monitoring = d.pop(
            "always_collect_error_samples_in_monitoring", UNSET
        )

        priority = d.pop("priority", UNSET)

        _owner = d.pop("owner", UNSET)
        owner: Union[Unset, TableOwnerSpec]
        if isinstance(_owner, Unset):
            owner = UNSET
        else:
            owner = TableOwnerSpec.from_dict(_owner)

        _profiling_checks_result_truncation = d.pop(
            "profiling_checks_result_truncation", UNSET
        )
        profiling_checks_result_truncation: Union[Unset, ProfilingTimePeriodTruncation]
        if isinstance(_profiling_checks_result_truncation, Unset):
            profiling_checks_result_truncation = UNSET
        else:
            profiling_checks_result_truncation = ProfilingTimePeriodTruncation(
                _profiling_checks_result_truncation
            )

        _file_format = d.pop("file_format", UNSET)
        file_format: Union[Unset, FileFormatSpec]
        if isinstance(_file_format, Unset):
            file_format = UNSET
        else:
            file_format = FileFormatSpec.from_dict(_file_format)

        _data_quality_status = d.pop("data_quality_status", UNSET)
        data_quality_status: Union[Unset, TableCurrentDataQualityStatusModel]
        if isinstance(_data_quality_status, Unset):
            data_quality_status = UNSET
        else:
            data_quality_status = TableCurrentDataQualityStatusModel.from_dict(
                _data_quality_status
            )

        has_any_configured_checks = d.pop("has_any_configured_checks", UNSET)

        has_any_configured_profiling_checks = d.pop(
            "has_any_configured_profiling_checks", UNSET
        )

        has_any_configured_monitoring_checks = d.pop(
            "has_any_configured_monitoring_checks", UNSET
        )

        has_any_configured_partition_checks = d.pop(
            "has_any_configured_partition_checks", UNSET
        )

        partitioning_configuration_missing = d.pop(
            "partitioning_configuration_missing", UNSET
        )

        _run_checks_job_template = d.pop("run_checks_job_template", UNSET)
        run_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_checks_job_template, Unset):
            run_checks_job_template = UNSET
        else:
            run_checks_job_template = CheckSearchFilters.from_dict(
                _run_checks_job_template
            )

        _run_profiling_checks_job_template = d.pop(
            "run_profiling_checks_job_template", UNSET
        )
        run_profiling_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_profiling_checks_job_template, Unset):
            run_profiling_checks_job_template = UNSET
        else:
            run_profiling_checks_job_template = CheckSearchFilters.from_dict(
                _run_profiling_checks_job_template
            )

        _run_monitoring_checks_job_template = d.pop(
            "run_monitoring_checks_job_template", UNSET
        )
        run_monitoring_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_monitoring_checks_job_template, Unset):
            run_monitoring_checks_job_template = UNSET
        else:
            run_monitoring_checks_job_template = CheckSearchFilters.from_dict(
                _run_monitoring_checks_job_template
            )

        _run_partition_checks_job_template = d.pop(
            "run_partition_checks_job_template", UNSET
        )
        run_partition_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_partition_checks_job_template, Unset):
            run_partition_checks_job_template = UNSET
        else:
            run_partition_checks_job_template = CheckSearchFilters.from_dict(
                _run_partition_checks_job_template
            )

        _collect_statistics_job_template = d.pop(
            "collect_statistics_job_template", UNSET
        )
        collect_statistics_job_template: Union[Unset, StatisticsCollectorSearchFilters]
        if isinstance(_collect_statistics_job_template, Unset):
            collect_statistics_job_template = UNSET
        else:
            collect_statistics_job_template = (
                StatisticsCollectorSearchFilters.from_dict(
                    _collect_statistics_job_template
                )
            )

        _data_clean_job_template = d.pop("data_clean_job_template", UNSET)
        data_clean_job_template: Union[Unset, DeleteStoredDataQueueJobParameters]
        if isinstance(_data_clean_job_template, Unset):
            data_clean_job_template = UNSET
        else:
            data_clean_job_template = DeleteStoredDataQueueJobParameters.from_dict(
                _data_clean_job_template
            )

        _advanced_properties = d.pop("advanced_properties", UNSET)
        advanced_properties: Union[Unset, TableListModelAdvancedProperties]
        if isinstance(_advanced_properties, Unset):
            advanced_properties = UNSET
        else:
            advanced_properties = TableListModelAdvancedProperties.from_dict(
                _advanced_properties
            )

        can_edit = d.pop("can_edit", UNSET)

        can_collect_statistics = d.pop("can_collect_statistics", UNSET)

        can_run_checks = d.pop("can_run_checks", UNSET)

        can_delete_data = d.pop("can_delete_data", UNSET)

        yaml_parsing_error = d.pop("yaml_parsing_error", UNSET)

        table_list_model = cls(
            connection_name=connection_name,
            table_hash=table_hash,
            target=target,
            labels=labels,
            disabled=disabled,
            stage=stage,
            filter_=filter_,
            do_not_collect_error_samples_in_profiling=do_not_collect_error_samples_in_profiling,
            always_collect_error_samples_in_monitoring=always_collect_error_samples_in_monitoring,
            priority=priority,
            owner=owner,
            profiling_checks_result_truncation=profiling_checks_result_truncation,
            file_format=file_format,
            data_quality_status=data_quality_status,
            has_any_configured_checks=has_any_configured_checks,
            has_any_configured_profiling_checks=has_any_configured_profiling_checks,
            has_any_configured_monitoring_checks=has_any_configured_monitoring_checks,
            has_any_configured_partition_checks=has_any_configured_partition_checks,
            partitioning_configuration_missing=partitioning_configuration_missing,
            run_checks_job_template=run_checks_job_template,
            run_profiling_checks_job_template=run_profiling_checks_job_template,
            run_monitoring_checks_job_template=run_monitoring_checks_job_template,
            run_partition_checks_job_template=run_partition_checks_job_template,
            collect_statistics_job_template=collect_statistics_job_template,
            data_clean_job_template=data_clean_job_template,
            advanced_properties=advanced_properties,
            can_edit=can_edit,
            can_collect_statistics=can_collect_statistics,
            can_run_checks=can_run_checks,
            can_delete_data=can_delete_data,
            yaml_parsing_error=yaml_parsing_error,
        )

        table_list_model.additional_properties = d
        return table_list_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.column_comparison_monthly_partitioned_checks_spec import (
        ColumnComparisonMonthlyPartitionedChecksSpec,
    )


T = TypeVar("T", bound="ColumnMonthlyPartitionedCheckCategoriesSpecComparisons")


@_attrs_define
class ColumnMonthlyPartitionedCheckCategoriesSpecComparisons:
    """Dictionary of configuration of checks for table comparisons at a column level. The key that identifies each
    comparison must match the name of a data comparison that is configured on the parent table.

    """

    additional_properties: Dict[str, "ColumnComparisonMonthlyPartitionedChecksSpec"] = (
        _attrs_field(init=False, factory=dict)
    )

    def to_dict(self) -> Dict[str, Any]:
        pass

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()

        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_comparison_monthly_partitioned_checks_spec import (
            ColumnComparisonMonthlyPartitionedChecksSpec,
        )

        d = src_dict.copy()
        column_monthly_partitioned_check_categories_spec_comparisons = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = (
                ColumnComparisonMonthlyPartitionedChecksSpec.from_dict(prop_dict)
            )

            additional_properties[prop_name] = additional_property

        column_monthly_partitioned_check_categories_spec_comparisons.additional_properties = (
            additional_properties
        )
        return column_monthly_partitioned_check_categories_spec_comparisons

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> "ColumnComparisonMonthlyPartitionedChecksSpec":
        return self.additional_properties[key]

    def __setitem__(
        self, key: str, value: "ColumnComparisonMonthlyPartitionedChecksSpec"
    ) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.check_result_status import CheckResultStatus
from ..models.check_time_scale import CheckTimeScale
from ..models.check_type import CheckType
from ..models.rule_severity_level import RuleSeverityLevel
from ..types import UNSET, Unset

T = TypeVar("T", bound="CheckCurrentDataQualityStatusModel")


@_attrs_define
class CheckCurrentDataQualityStatusModel:
    """The most recent data quality status for a single data quality check. If data grouping is enabled, the
    *current_severity* will be the highest data quality issue status from all data quality results for all data groups.
    For partitioned checks, it is the highest severity of all results for all partitions (time periods) in the analyzed
    time range.

        Attributes:
            current_severity (Union[Unset, CheckResultStatus]):
            highest_historical_severity (Union[Unset, RuleSeverityLevel]):
            last_executed_at (Union[Unset, int]): The UTC timestamp when the check was recently executed.
            column_name (Union[Unset, str]): Optional column name for column-level data quality checks.
            check_type (Union[Unset, CheckType]):
            time_scale (Union[Unset, CheckTimeScale]):
            category (Union[Unset, str]): Check category name, such as nulls, schema, strings, volume.
            quality_dimension (Union[Unset, str]): Data quality dimension, such as Completeness, Uniqueness, Validity.
            executed_checks (Union[Unset, int]): The total number of most recent checks that were executed on the column.
                Table comparison checks that are comparing groups of data are counted as the number of compared data groups.
            valid_results (Union[Unset, int]): The number of most recent valid data quality checks that passed without
                raising any issues.
            warnings (Union[Unset, int]): The number of most recent data quality checks that failed by raising a warning
                severity data quality issue.
            errors (Union[Unset, int]): The number of most recent data quality checks that failed by raising an error
                severity data quality issue.
            fatals (Union[Unset, int]): The number of most recent data quality checks that failed by raising a fatal
                severity data quality issue.
            execution_errors (Union[Unset, int]): The number of data quality check execution errors that were reported due
                to access issues to the data source, invalid mapping in DQOps, invalid queries in data quality sensors or
                invalid python rules. When an execution error is reported, the configuration of a data quality check on a column
                must be updated.
    """

    current_severity: Union[Unset, CheckResultStatus] = UNSET
    highest_historical_severity: Union[Unset, RuleSeverityLevel] = UNSET
    last_executed_at: Union[Unset, int] = UNSET
    column_name: Union[Unset, str] = UNSET
    check_type: Union[Unset, CheckType] = UNSET
    time_scale: Union[Unset, CheckTimeScale] = UNSET
    category: Union[Unset, str] = UNSET
    quality_dimension: Union[Unset, str] = UNSET
    executed_checks: Union[Unset, int] = UNSET
    valid_results: Union[Unset, int] = UNSET
    warnings: Union[Unset, int] = UNSET
    errors: Union[Unset, int] = UNSET
    fatals: Union[Unset, int] = UNSET
    execution_errors: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        current_severity: Union[Unset, str] = UNSET
        if not isinstance(self.current_severity, Unset):
            current_severity = self.current_severity.value

        highest_historical_severity: Union[Unset, str] = UNSET
        if not isinstance(self.highest_historical_severity, Unset):
            highest_historical_severity = self.highest_historical_severity.value

        last_executed_at = self.last_executed_at
        column_name = self.column_name
        check_type: Union[Unset, str] = UNSET
        if not isinstance(self.check_type, Unset):
            check_type = self.check_type.value

        time_scale: Union[Unset, str] = UNSET
        if not isinstance(self.time_scale, Unset):
            time_scale = self.time_scale.value

        category = self.category
        quality_dimension = self.quality_dimension
        executed_checks = self.executed_checks
        valid_results = self.valid_results
        warnings = self.warnings
        errors = self.errors
        fatals = self.fatals
        execution_errors = self.execution_errors

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if current_severity is not UNSET:
            field_dict["current_severity"] = current_severity
        if highest_historical_severity is not UNSET:
            field_dict["highest_historical_severity"] = highest_historical_severity
        if last_executed_at is not UNSET:
            field_dict["last_executed_at"] = last_executed_at
        if column_name is not UNSET:
            field_dict["column_name"] = column_name
        if check_type is not UNSET:
            field_dict["check_type"] = check_type
        if time_scale is not UNSET:
            field_dict["time_scale"] = time_scale
        if category is not UNSET:
            field_dict["category"] = category
        if quality_dimension is not UNSET:
            field_dict["quality_dimension"] = quality_dimension
        if executed_checks is not UNSET:
            field_dict["executed_checks"] = executed_checks
        if valid_results is not UNSET:
            field_dict["valid_results"] = valid_results
        if warnings is not UNSET:
            field_dict["warnings"] = warnings
        if errors is not UNSET:
            field_dict["errors"] = errors
        if fatals is not UNSET:
            field_dict["fatals"] = fatals
        if execution_errors is not UNSET:
            field_dict["execution_errors"] = execution_errors

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _current_severity = d.pop("current_severity", UNSET)
        current_severity: Union[Unset, CheckResultStatus]
        if isinstance(_current_severity, Unset):
            current_severity = UNSET
        else:
            current_severity = CheckResultStatus(_current_severity)

        _highest_historical_severity = d.pop("highest_historical_severity", UNSET)
        highest_historical_severity: Union[Unset, RuleSeverityLevel]
        if isinstance(_highest_historical_severity, Unset):
            highest_historical_severity = UNSET
        else:
            highest_historical_severity = RuleSeverityLevel(
                _highest_historical_severity
            )

        last_executed_at = d.pop("last_executed_at", UNSET)

        column_name = d.pop("column_name", UNSET)

        _check_type = d.pop("check_type", UNSET)
        check_type: Union[Unset, CheckType]
        if isinstance(_check_type, Unset):
            check_type = UNSET
        else:
            check_type = CheckType(_check_type)

        _time_scale = d.pop("time_scale", UNSET)
        time_scale: Union[Unset, CheckTimeScale]
        if isinstance(_time_scale, Unset):
            time_scale = UNSET
        else:
            time_scale = CheckTimeScale(_time_scale)

        category = d.pop("category", UNSET)

        quality_dimension = d.pop("quality_dimension", UNSET)

        executed_checks = d.pop("executed_checks", UNSET)

        valid_results = d.pop("valid_results", UNSET)

        warnings = d.pop("warnings", UNSET)

        errors = d.pop("errors", UNSET)

        fatals = d.pop("fatals", UNSET)

        execution_errors = d.pop("execution_errors", UNSET)

        check_current_data_quality_status_model = cls(
            current_severity=current_severity,
            highest_historical_severity=highest_historical_severity,
            last_executed_at=last_executed_at,
            column_name=column_name,
            check_type=check_type,
            time_scale=time_scale,
            category=category,
            quality_dimension=quality_dimension,
            executed_checks=executed_checks,
            valid_results=valid_results,
            warnings=warnings,
            errors=errors,
            fatals=fatals,
            execution_errors=execution_errors,
        )

        check_current_data_quality_status_model.additional_properties = d
        return check_current_data_quality_status_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.provider_type import ProviderType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.auto_import_tables_spec import AutoImportTablesSpec
    from ..models.big_query_parameters_spec import BigQueryParametersSpec
    from ..models.click_house_parameters_spec import ClickHouseParametersSpec
    from ..models.comment_spec import CommentSpec
    from ..models.connection_incident_grouping_spec import (
        ConnectionIncidentGroupingSpec,
    )
    from ..models.connection_spec_advanced_properties import (
        ConnectionSpecAdvancedProperties,
    )
    from ..models.cron_schedules_spec import CronSchedulesSpec
    from ..models.data_grouping_configuration_spec import DataGroupingConfigurationSpec
    from ..models.databricks_parameters_spec import DatabricksParametersSpec
    from ..models.db_2_parameters_spec import Db2ParametersSpec
    from ..models.duckdb_parameters_spec import DuckdbParametersSpec
    from ..models.hana_parameters_spec import HanaParametersSpec
    from ..models.maria_db_parameters_spec import MariaDbParametersSpec
    from ..models.mysql_parameters_spec import MysqlParametersSpec
    from ..models.oracle_parameters_spec import OracleParametersSpec
    from ..models.postgresql_parameters_spec import PostgresqlParametersSpec
    from ..models.presto_parameters_spec import PrestoParametersSpec
    from ..models.quest_db_parameters_spec import QuestDbParametersSpec
    from ..models.redshift_parameters_spec import RedshiftParametersSpec
    from ..models.snowflake_parameters_spec import SnowflakeParametersSpec
    from ..models.spark_parameters_spec import SparkParametersSpec
    from ..models.sql_server_parameters_spec import SqlServerParametersSpec
    from ..models.teradata_parameters_spec import TeradataParametersSpec
    from ..models.trino_parameters_spec import TrinoParametersSpec


T = TypeVar("T", bound="ConnectionSpec")


@_attrs_define
class ConnectionSpec:
    """
    Attributes:
        provider_type (Union[Unset, ProviderType]):
        bigquery (Union[Unset, BigQueryParametersSpec]):
        snowflake (Union[Unset, SnowflakeParametersSpec]):
        postgresql (Union[Unset, PostgresqlParametersSpec]):
        duckdb (Union[Unset, DuckdbParametersSpec]):
        redshift (Union[Unset, RedshiftParametersSpec]):
        sqlserver (Union[Unset, SqlServerParametersSpec]):
        presto (Union[Unset, PrestoParametersSpec]):
        trino (Union[Unset, TrinoParametersSpec]):
        mysql (Union[Unset, MysqlParametersSpec]):
        oracle (Union[Unset, OracleParametersSpec]):
        spark (Union[Unset, SparkParametersSpec]):
        databricks (Union[Unset, DatabricksParametersSpec]):
        hana (Union[Unset, HanaParametersSpec]):
        db2 (Union[Unset, Db2ParametersSpec]):
        mariadb (Union[Unset, MariaDbParametersSpec]):
        clickhouse (Union[Unset, ClickHouseParametersSpec]):
        questdb (Union[Unset, QuestDbParametersSpec]):
        teradata (Union[Unset, TeradataParametersSpec]):
        parallel_jobs_limit (Union[Unset, int]): The concurrency limit for the maximum number of parallel SQL queries
            executed on this connection.
        default_grouping_configuration (Union[Unset, DataGroupingConfigurationSpec]):
        schedules (Union[Unset, CronSchedulesSpec]):
        auto_import_tables (Union[Unset, AutoImportTablesSpec]):
        schedule_on_instance (Union[Unset, str]): Limits running scheduled checks (started by a CRON job scheduler) to
            run only on a named DQOps instance. When this field is empty, data quality checks are run on all DQOps
            instances. Set a DQOps instance name to run checks on a named instance only. The default name of the DQOps Cloud
            SaaS instance is "cloud".
        incident_grouping (Union[Unset, ConnectionIncidentGroupingSpec]):
        comments (Union[Unset, List['CommentSpec']]): Comments for change tracking. Please put comments in this
            collection because YAML comments may be removed when the YAML file is modified by the tool (serialization and
            deserialization will remove non tracked comments).
        labels (Union[Unset, List[str]]): Custom labels that were assigned to the connection. Labels are used for
            searching for tables when filtered data quality checks are executed.
        advanced_properties (Union[Unset, ConnectionSpecAdvancedProperties]): A dictionary of advanced properties that
            can be used for e.g. to support mapping data to data catalogs, a key/value dictionary.
    """

    provider_type: Union[Unset, ProviderType] = UNSET
    bigquery: Union[Unset, "BigQueryParametersSpec"] = UNSET
    snowflake: Union[Unset, "SnowflakeParametersSpec"] = UNSET
    postgresql: Union[Unset, "PostgresqlParametersSpec"] = UNSET
    duckdb: Union[Unset, "DuckdbParametersSpec"] = UNSET
    redshift: Union[Unset, "RedshiftParametersSpec"] = UNSET
    sqlserver: Union[Unset, "SqlServerParametersSpec"] = UNSET
    presto: Union[Unset, "PrestoParametersSpec"] = UNSET
    trino: Union[Unset, "TrinoParametersSpec"] = UNSET
    mysql: Union[Unset, "MysqlParametersSpec"] = UNSET
    oracle: Union[Unset, "OracleParametersSpec"] = UNSET
    spark: Union[Unset, "SparkParametersSpec"] = UNSET
    databricks: Union[Unset, "DatabricksParametersSpec"] = UNSET
    hana: Union[Unset, "HanaParametersSpec"] = UNSET
    db2: Union[Unset, "Db2ParametersSpec"] = UNSET
    mariadb: Union[Unset, "MariaDbParametersSpec"] = UNSET
    clickhouse: Union[Unset, "ClickHouseParametersSpec"] = UNSET
    questdb: Union[Unset, "QuestDbParametersSpec"] = UNSET
    teradata: Union[Unset, "TeradataParametersSpec"] = UNSET
    parallel_jobs_limit: Union[Unset, int] = UNSET
    default_grouping_configuration: Union[Unset, "DataGroupingConfigurationSpec"] = (
        UNSET
    )
    schedules: Union[Unset, "CronSchedulesSpec"] = UNSET
    auto_import_tables: Union[Unset, "AutoImportTablesSpec"] = UNSET
    schedule_on_instance: Union[Unset, str] = UNSET
    incident_grouping: Union[Unset, "ConnectionIncidentGroupingSpec"] = UNSET
    comments: Union[Unset, List["CommentSpec"]] = UNSET
    labels: Union[Unset, List[str]] = UNSET
    advanced_properties: Union[Unset, "ConnectionSpecAdvancedProperties"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        provider_type: Union[Unset, str] = UNSET
        if not isinstance(self.provider_type, Unset):
            provider_type = self.provider_type.value

        bigquery: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bigquery, Unset):
            bigquery = self.bigquery.to_dict()

        snowflake: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.snowflake, Unset):
            snowflake = self.snowflake.to_dict()

        postgresql: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.postgresql, Unset):
            postgresql = self.postgresql.to_dict()

        duckdb: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.duckdb, Unset):
            duckdb = self.duckdb.to_dict()

        redshift: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.redshift, Unset):
            redshift = self.redshift.to_dict()

        sqlserver: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sqlserver, Unset):
            sqlserver = self.sqlserver.to_dict()

        presto: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.presto, Unset):
            presto = self.presto.to_dict()

        trino: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trino, Unset):
            trino = self.trino.to_dict()

        mysql: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.mysql, Unset):
            mysql = self.mysql.to_dict()

        oracle: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.oracle, Unset):
            oracle = self.oracle.to_dict()

        spark: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.spark, Unset):
            spark = self.spark.to_dict()

        databricks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.databricks, Unset):
            databricks = self.databricks.to_dict()

        hana: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.hana, Unset):
            hana = self.hana.to_dict()

        db2: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.db2, Unset):
            db2 = self.db2.to_dict()

        mariadb: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.mariadb, Unset):
            mariadb = self.mariadb.to_dict()

        clickhouse: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.clickhouse, Unset):
            clickhouse = self.clickhouse.to_dict()

        questdb: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.questdb, Unset):
            questdb = self.questdb.to_dict()

        teradata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.teradata, Unset):
            teradata = self.teradata.to_dict()

        parallel_jobs_limit = self.parallel_jobs_limit
        default_grouping_configuration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default_grouping_configuration, Unset):
            default_grouping_configuration = (
                self.default_grouping_configuration.to_dict()
            )

        schedules: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.schedules, Unset):
            schedules = self.schedules.to_dict()

        auto_import_tables: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.auto_import_tables, Unset):
            auto_import_tables = self.auto_import_tables.to_dict()

        schedule_on_instance = self.schedule_on_instance
        incident_grouping: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.incident_grouping, Unset):
            incident_grouping = self.incident_grouping.to_dict()

        comments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.comments, Unset):
            comments = []
            for comments_item_data in self.comments:
                comments_item = comments_item_data.to_dict()

                comments.append(comments_item)

        labels: Union[Unset, List[str]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels

        advanced_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.advanced_properties, Unset):
            advanced_properties = self.advanced_properties.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if provider_type is not UNSET:
            field_dict["provider_type"] = provider_type
        if bigquery is not UNSET:
            field_dict["bigquery"] = bigquery
        if snowflake is not UNSET:
            field_dict["snowflake"] = snowflake
        if postgresql is not UNSET:
            field_dict["postgresql"] = postgresql
        if duckdb is not UNSET:
            field_dict["duckdb"] = duckdb
        if redshift is not UNSET:
            field_dict["redshift"] = redshift
        if sqlserver is not UNSET:
            field_dict["sqlserver"] = sqlserver
        if presto is not UNSET:
            field_dict["presto"] = presto
        if trino is not UNSET:
            field_dict["trino"] = trino
        if mysql is not UNSET:
            field_dict["mysql"] = mysql
        if oracle is not UNSET:
            field_dict["oracle"] = oracle
        if spark is not UNSET:
            field_dict["spark"] = spark
        if databricks is not UNSET:
            field_dict["databricks"] = databricks
        if hana is not UNSET:
            field_dict["hana"] = hana
        if db2 is not UNSET:
            field_dict["db2"] = db2
        if mariadb is not UNSET:
            field_dict["mariadb"] = mariadb
        if clickhouse is not UNSET:
            field_dict["clickhouse"] = clickhouse
        if questdb is not UNSET:
            field_dict["questdb"] = questdb
        if teradata is not UNSET:
            field_dict["teradata"] = teradata
        if parallel_jobs_limit is not UNSET:
            field_dict["parallel_jobs_limit"] = parallel_jobs_limit
        if default_grouping_configuration is not UNSET:
            field_dict["default_grouping_configuration"] = (
                default_grouping_configuration
            )
        if schedules is not UNSET:
            field_dict["schedules"] = schedules
        if auto_import_tables is not UNSET:
            field_dict["auto_import_tables"] = auto_import_tables
        if schedule_on_instance is not UNSET:
            field_dict["schedule_on_instance"] = schedule_on_instance
        if incident_grouping is not UNSET:
            field_dict["incident_grouping"] = incident_grouping
        if comments is not UNSET:
            field_dict["comments"] = comments
        if labels is not UNSET:
            field_dict["labels"] = labels
        if advanced_properties is not UNSET:
            field_dict["advanced_properties"] = advanced_properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.auto_import_tables_spec import AutoImportTablesSpec
        from ..models.big_query_parameters_spec import BigQueryParametersSpec
        from ..models.click_house_parameters_spec import ClickHouseParametersSpec
        from ..models.comment_spec import CommentSpec
        from ..models.connection_incident_grouping_spec import (
            ConnectionIncidentGroupingSpec,
        )
        from ..models.connection_spec_advanced_properties import (
            ConnectionSpecAdvancedProperties,
        )
        from ..models.cron_schedules_spec import CronSchedulesSpec
        from ..models.data_grouping_configuration_spec import (
            DataGroupingConfigurationSpec,
        )
        from ..models.databricks_parameters_spec import DatabricksParametersSpec
        from ..models.db_2_parameters_spec import Db2ParametersSpec
        from ..models.duckdb_parameters_spec import DuckdbParametersSpec
        from ..models.hana_parameters_spec import HanaParametersSpec
        from ..models.maria_db_parameters_spec import MariaDbParametersSpec
        from ..models.mysql_parameters_spec import MysqlParametersSpec
        from ..models.oracle_parameters_spec import OracleParametersSpec
        from ..models.postgresql_parameters_spec import PostgresqlParametersSpec
        from ..models.presto_parameters_spec import PrestoParametersSpec
        from ..models.quest_db_parameters_spec import QuestDbParametersSpec
        from ..models.redshift_parameters_spec import RedshiftParametersSpec
        from ..models.snowflake_parameters_spec import SnowflakeParametersSpec
        from ..models.spark_parameters_spec import SparkParametersSpec
        from ..models.sql_server_parameters_spec import SqlServerParametersSpec
        from ..models.teradata_parameters_spec import TeradataParametersSpec
        from ..models.trino_parameters_spec import TrinoParametersSpec

        d = src_dict.copy()
        _provider_type = d.pop("provider_type", UNSET)
        provider_type: Union[Unset, ProviderType]
        if isinstance(_provider_type, Unset):
            provider_type = UNSET
        else:
            provider_type = ProviderType(_provider_type)

        _bigquery = d.pop("bigquery", UNSET)
        bigquery: Union[Unset, BigQueryParametersSpec]
        if isinstance(_bigquery, Unset):
            bigquery = UNSET
        else:
            bigquery = BigQueryParametersSpec.from_dict(_bigquery)

        _snowflake = d.pop("snowflake", UNSET)
        snowflake: Union[Unset, SnowflakeParametersSpec]
        if isinstance(_snowflake, Unset):
            snowflake = UNSET
        else:
            snowflake = SnowflakeParametersSpec.from_dict(_snowflake)

        _postgresql = d.pop("postgresql", UNSET)
        postgresql: Union[Unset, PostgresqlParametersSpec]
        if isinstance(_postgresql, Unset):
            postgresql = UNSET
        else:
            postgresql = PostgresqlParametersSpec.from_dict(_postgresql)

        _duckdb = d.pop("duckdb", UNSET)
        duckdb: Union[Unset, DuckdbParametersSpec]
        if isinstance(_duckdb, Unset):
            duckdb = UNSET
        else:
            duckdb = DuckdbParametersSpec.from_dict(_duckdb)

        _redshift = d.pop("redshift", UNSET)
        redshift: Union[Unset, RedshiftParametersSpec]
        if isinstance(_redshift, Unset):
            redshift = UNSET
        else:
            redshift = RedshiftParametersSpec.from_dict(_redshift)

        _sqlserver = d.pop("sqlserver", UNSET)
        sqlserver: Union[Unset, SqlServerParametersSpec]
        if isinstance(_sqlserver, Unset):
            sqlserver = UNSET
        else:
            sqlserver = SqlServerParametersSpec.from_dict(_sqlserver)

        _presto = d.pop("presto", UNSET)
        presto: Union[Unset, PrestoParametersSpec]
        if isinstance(_presto, Unset):
            presto = UNSET
        else:
            presto = PrestoParametersSpec.from_dict(_presto)

        _trino = d.pop("trino", UNSET)
        trino: Union[Unset, TrinoParametersSpec]
        if isinstance(_trino, Unset):
            trino = UNSET
        else:
            trino = TrinoParametersSpec.from_dict(_trino)

        _mysql = d.pop("mysql", UNSET)
        mysql: Union[Unset, MysqlParametersSpec]
        if isinstance(_mysql, Unset):
            mysql = UNSET
        else:
            mysql = MysqlParametersSpec.from_dict(_mysql)

        _oracle = d.pop("oracle", UNSET)
        oracle: Union[Unset, OracleParametersSpec]
        if isinstance(_oracle, Unset):
            oracle = UNSET
        else:
            oracle = OracleParametersSpec.from_dict(_oracle)

        _spark = d.pop("spark", UNSET)
        spark: Union[Unset, SparkParametersSpec]
        if isinstance(_spark, Unset):
            spark = UNSET
        else:
            spark = SparkParametersSpec.from_dict(_spark)

        _databricks = d.pop("databricks", UNSET)
        databricks: Union[Unset, DatabricksParametersSpec]
        if isinstance(_databricks, Unset):
            databricks = UNSET
        else:
            databricks = DatabricksParametersSpec.from_dict(_databricks)

        _hana = d.pop("hana", UNSET)
        hana: Union[Unset, HanaParametersSpec]
        if isinstance(_hana, Unset):
            hana = UNSET
        else:
            hana = HanaParametersSpec.from_dict(_hana)

        _db2 = d.pop("db2", UNSET)
        db2: Union[Unset, Db2ParametersSpec]
        if isinstance(_db2, Unset):
            db2 = UNSET
        else:
            db2 = Db2ParametersSpec.from_dict(_db2)

        _mariadb = d.pop("mariadb", UNSET)
        mariadb: Union[Unset, MariaDbParametersSpec]
        if isinstance(_mariadb, Unset):
            mariadb = UNSET
        else:
            mariadb = MariaDbParametersSpec.from_dict(_mariadb)

        _clickhouse = d.pop("clickhouse", UNSET)
        clickhouse: Union[Unset, ClickHouseParametersSpec]
        if isinstance(_clickhouse, Unset):
            clickhouse = UNSET
        else:
            clickhouse = ClickHouseParametersSpec.from_dict(_clickhouse)

        _questdb = d.pop("questdb", UNSET)
        questdb: Union[Unset, QuestDbParametersSpec]
        if isinstance(_questdb, Unset):
            questdb = UNSET
        else:
            questdb = QuestDbParametersSpec.from_dict(_questdb)

        _teradata = d.pop("teradata", UNSET)
        teradata: Union[Unset, TeradataParametersSpec]
        if isinstance(_teradata, Unset):
            teradata = UNSET
        else:
            teradata = TeradataParametersSpec.from_dict(_teradata)

        parallel_jobs_limit = d.pop("parallel_jobs_limit", UNSET)

        _default_grouping_configuration = d.pop("default_grouping_configuration", UNSET)
        default_grouping_configuration: Union[Unset, DataGroupingConfigurationSpec]
        if isinstance(_default_grouping_configuration, Unset):
            default_grouping_configuration = UNSET
        else:
            default_grouping_configuration = DataGroupingConfigurationSpec.from_dict(
                _default_grouping_configuration
            )

        _schedules = d.pop("schedules", UNSET)
        schedules: Union[Unset, CronSchedulesSpec]
        if isinstance(_schedules, Unset):
            schedules = UNSET
        else:
            schedules = CronSchedulesSpec.from_dict(_schedules)

        _auto_import_tables = d.pop("auto_import_tables", UNSET)
        auto_import_tables: Union[Unset, AutoImportTablesSpec]
        if isinstance(_auto_import_tables, Unset):
            auto_import_tables = UNSET
        else:
            auto_import_tables = AutoImportTablesSpec.from_dict(_auto_import_tables)

        schedule_on_instance = d.pop("schedule_on_instance", UNSET)

        _incident_grouping = d.pop("incident_grouping", UNSET)
        incident_grouping: Union[Unset, ConnectionIncidentGroupingSpec]
        if isinstance(_incident_grouping, Unset):
            incident_grouping = UNSET
        else:
            incident_grouping = ConnectionIncidentGroupingSpec.from_dict(
                _incident_grouping
            )

        comments = []
        _comments = d.pop("comments", UNSET)
        for comments_item_data in _comments or []:
            comments_item = CommentSpec.from_dict(comments_item_data)

            comments.append(comments_item)

        labels = cast(List[str], d.pop("labels", UNSET))

        _advanced_properties = d.pop("advanced_properties", UNSET)
        advanced_properties: Union[Unset, ConnectionSpecAdvancedProperties]
        if isinstance(_advanced_properties, Unset):
            advanced_properties = UNSET
        else:
            advanced_properties = ConnectionSpecAdvancedProperties.from_dict(
                _advanced_properties
            )

        connection_spec = cls(
            provider_type=provider_type,
            bigquery=bigquery,
            snowflake=snowflake,
            postgresql=postgresql,
            duckdb=duckdb,
            redshift=redshift,
            sqlserver=sqlserver,
            presto=presto,
            trino=trino,
            mysql=mysql,
            oracle=oracle,
            spark=spark,
            databricks=databricks,
            hana=hana,
            db2=db2,
            mariadb=mariadb,
            clickhouse=clickhouse,
            questdb=questdb,
            teradata=teradata,
            parallel_jobs_limit=parallel_jobs_limit,
            default_grouping_configuration=default_grouping_configuration,
            schedules=schedules,
            auto_import_tables=auto_import_tables,
            schedule_on_instance=schedule_on_instance,
            incident_grouping=incident_grouping,
            comments=comments,
            labels=labels,
            advanced_properties=advanced_properties,
        )

        connection_spec.additional_properties = d
        return connection_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ChangePercent1DayRule10ParametersSpec")


@_attrs_define
class ChangePercent1DayRule10ParametersSpec:
    """
    Attributes:
        max_percent (Union[Unset, float]): Percentage of maximum accepted change compared to a readout 1 day ago
            (inclusive).
        exact_day (Union[Unset, bool]): When the exact_day parameter is unchecked (exact_day: false), the rule search
            for the most recent sensor readouts from the past 60 days and compares them. If the parameter is selected
            (exact_day: true), the rule compares only with the results from the past 1 day. If no results are found from
            that time, no results or errors will be generated.
    """

    max_percent: Union[Unset, float] = UNSET
    exact_day: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        max_percent = self.max_percent
        exact_day = self.exact_day

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if max_percent is not UNSET:
            field_dict["max_percent"] = max_percent
        if exact_day is not UNSET:
            field_dict["exact_day"] = exact_day

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        max_percent = d.pop("max_percent", UNSET)

        exact_day = d.pop("exact_day", UNSET)

        change_percent_1_day_rule_10_parameters_spec = cls(
            max_percent=max_percent,
            exact_day=exact_day,
        )

        change_percent_1_day_rule_10_parameters_spec.additional_properties = d
        return change_percent_1_day_rule_10_parameters_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ParameterDataType(str, Enum):
    BOOLEAN = "boolean"
    COLUMN_NAME = "column_name"
    DATE = "date"
    DATETIME = "datetime"
    DOUBLE = "double"
    ENUM = "enum"
    INTEGER = "integer"
    INTEGER_LIST = "integer_list"
    LONG = "long"
    OBJECT = "object"
    STRING = "string"
    STRING_LIST = "string_list"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.connection_test_status import ConnectionTestStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ConnectionTestModel")


@_attrs_define
class ConnectionTestModel:
    """Connection test status result

    Attributes:
        connection_test_result (Union[Unset, ConnectionTestStatus]):
        error_message (Union[Unset, str]): Optional error message when the status is not "SUCCESS"
    """

    connection_test_result: Union[Unset, ConnectionTestStatus] = UNSET
    error_message: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connection_test_result: Union[Unset, str] = UNSET
        if not isinstance(self.connection_test_result, Unset):
            connection_test_result = self.connection_test_result.value

        error_message = self.error_message

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connection_test_result is not UNSET:
            field_dict["connectionTestResult"] = connection_test_result
        if error_message is not UNSET:
            field_dict["errorMessage"] = error_message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _connection_test_result = d.pop("connectionTestResult", UNSET)
        connection_test_result: Union[Unset, ConnectionTestStatus]
        if isinstance(_connection_test_result, Unset):
            connection_test_result = UNSET
        else:
            connection_test_result = ConnectionTestStatus(_connection_test_result)

        error_message = d.pop("errorMessage", UNSET)

        connection_test_model = cls(
            connection_test_result=connection_test_result,
            error_message=error_message,
        )

        connection_test_model.additional_properties = d
        return connection_test_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CronScheduleSpec")


@_attrs_define
class CronScheduleSpec:
    """
    Attributes:
        cron_expression (Union[Unset, str]): Unix style cron expression that specifies when to execute scheduled
            operations like running data quality checks or synchronizing the configuration with the cloud.
        disabled (Union[Unset, bool]): Disables the schedule. When the value of this 'disable' field is false, the
            schedule is stored in the metadata but it is not activated to run data quality checks.
    """

    cron_expression: Union[Unset, str] = UNSET
    disabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        cron_expression = self.cron_expression
        disabled = self.disabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cron_expression is not UNSET:
            field_dict["cron_expression"] = cron_expression
        if disabled is not UNSET:
            field_dict["disabled"] = disabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        cron_expression = d.pop("cron_expression", UNSET)

        disabled = d.pop("disabled", UNSET)

        cron_schedule_spec = cls(
            cron_expression=cron_expression,
            disabled=disabled,
        )

        cron_schedule_spec.additional_properties = d
        return cron_schedule_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.incident_model import IncidentModel
from ...models.incident_sort_order import IncidentSortOrder
from ...models.sort_direction import SortDirection
from ...types import UNSET, Response, Unset


def _get_kwargs(
    connection_name: str,
    *,
    months: Union[Unset, None, int] = UNSET,
    open_: Union[Unset, None, bool] = UNSET,
    acknowledged: Union[Unset, None, bool] = UNSET,
    resolved: Union[Unset, None, bool] = UNSET,
    muted: Union[Unset, None, bool] = UNSET,
    severity: Union[Unset, None, int] = UNSET,
    page: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    filter_: Union[Unset, None, str] = UNSET,
    dimension: Union[Unset, None, str] = UNSET,
    category: Union[Unset, None, str] = UNSET,
    order: Union[Unset, None, IncidentSortOrder] = UNSET,
    direction: Union[Unset, None, SortDirection] = UNSET,
) -> Dict[str, Any]:

    pass

    params: Dict[str, Any] = {}
    params["months"] = months

    params["open"] = open_

    params["acknowledged"] = acknowledged

    params["resolved"] = resolved

    params["muted"] = muted

    params["severity"] = severity

    params["page"] = page

    params["limit"] = limit

    params["filter"] = filter_

    params["dimension"] = dimension

    params["category"] = category

    json_order: Union[Unset, None, str] = UNSET
    if not isinstance(order, Unset):
        json_order = order.value if order else None

    params["order"] = json_order

    json_direction: Union[Unset, None, str] = UNSET
    if not isinstance(direction, Unset):
        json_direction = direction.value if direction else None

    params["direction"] = json_direction

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "api/incidents/{connectionName}".format(
            connectionName=connection_name,
        ),
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[List["IncidentModel"]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = IncidentModel.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[List["IncidentModel"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    connection_name: str,
    *,
    client: AuthenticatedClient,
    months: Union[Unset, None, int] = UNSET,
    open_: Union[Unset, None, bool] = UNSET,
    acknowledged: Union[Unset, None, bool] = UNSET,
    resolved: Union[Unset, None, bool] = UNSET,
    muted: Union[Unset, None, bool] = UNSET,
    severity: Union[Unset, None, int] = UNSET,
    page: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    filter_: Union[Unset, None, str] = UNSET,
    dimension: Union[Unset, None, str] = UNSET,
    category: Union[Unset, None, str] = UNSET,
    order: Union[Unset, None, IncidentSortOrder] = UNSET,
    direction: Union[Unset, None, SortDirection] = UNSET,
) -> Response[List["IncidentModel"]]:
    """findRecentIncidentsOnConnection

     Returns a list of recent data quality incidents.

    Args:
        connection_name (str):
        months (Union[Unset, None, int]):
        open_ (Union[Unset, None, bool]):
        acknowledged (Union[Unset, None, bool]):
        resolved (Union[Unset, None, bool]):
        muted (Union[Unset, None, bool]):
        severity (Union[Unset, None, int]):
        page (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        filter_ (Union[Unset, None, str]):
        dimension (Union[Unset, None, str]):
        category (Union[Unset, None, str]):
        order (Union[Unset, None, IncidentSortOrder]):
        direction (Union[Unset, None, SortDirection]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['IncidentModel']]
    """

    kwargs = _get_kwargs(
        connection_name=connection_name,
        months=months,
        open_=open_,
        acknowledged=acknowledged,
        resolved=resolved,
        muted=muted,
        severity=severity,
        page=page,
        limit=limit,
        filter_=filter_,
        dimension=dimension,
        category=category,
        order=order,
        direction=direction,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    connection_name: str,
    *,
    client: AuthenticatedClient,
    months: Union[Unset, None, int] = UNSET,
    open_: Union[Unset, None, bool] = UNSET,
    acknowledged: Union[Unset, None, bool] = UNSET,
    resolved: Union[Unset, None, bool] = UNSET,
    muted: Union[Unset, None, bool] = UNSET,
    severity: Union[Unset, None, int] = UNSET,
    page: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    filter_: Union[Unset, None, str] = UNSET,
    dimension: Union[Unset, None, str] = UNSET,
    category: Union[Unset, None, str] = UNSET,
    order: Union[Unset, None, IncidentSortOrder] = UNSET,
    direction: Union[Unset, None, SortDirection] = UNSET,
) -> Optional[List["IncidentModel"]]:
    """findRecentIncidentsOnConnection

     Returns a list of recent data quality incidents.

    Args:
        connection_name (str):
        months (Union[Unset, None, int]):
        open_ (Union[Unset, None, bool]):
        acknowledged (Union[Unset, None, bool]):
        resolved (Union[Unset, None, bool]):
        muted (Union[Unset, None, bool]):
        severity (Union[Unset, None, int]):
        page (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        filter_ (Union[Unset, None, str]):
        dimension (Union[Unset, None, str]):
        category (Union[Unset, None, str]):
        order (Union[Unset, None, IncidentSortOrder]):
        direction (Union[Unset, None, SortDirection]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['IncidentModel']
    """

    return sync_detailed(
        connection_name=connection_name,
        client=client,
        months=months,
        open_=open_,
        acknowledged=acknowledged,
        resolved=resolved,
        muted=muted,
        severity=severity,
        page=page,
        limit=limit,
        filter_=filter_,
        dimension=dimension,
        category=category,
        order=order,
        direction=direction,
    ).parsed


async def asyncio_detailed(
    connection_name: str,
    *,
    client: AuthenticatedClient,
    months: Union[Unset, None, int] = UNSET,
    open_: Union[Unset, None, bool] = UNSET,
    acknowledged: Union[Unset, None, bool] = UNSET,
    resolved: Union[Unset, None, bool] = UNSET,
    muted: Union[Unset, None, bool] = UNSET,
    severity: Union[Unset, None, int] = UNSET,
    page: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    filter_: Union[Unset, None, str] = UNSET,
    dimension: Union[Unset, None, str] = UNSET,
    category: Union[Unset, None, str] = UNSET,
    order: Union[Unset, None, IncidentSortOrder] = UNSET,
    direction: Union[Unset, None, SortDirection] = UNSET,
) -> Response[List["IncidentModel"]]:
    """findRecentIncidentsOnConnection

     Returns a list of recent data quality incidents.

    Args:
        connection_name (str):
        months (Union[Unset, None, int]):
        open_ (Union[Unset, None, bool]):
        acknowledged (Union[Unset, None, bool]):
        resolved (Union[Unset, None, bool]):
        muted (Union[Unset, None, bool]):
        severity (Union[Unset, None, int]):
        page (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        filter_ (Union[Unset, None, str]):
        dimension (Union[Unset, None, str]):
        category (Union[Unset, None, str]):
        order (Union[Unset, None, IncidentSortOrder]):
        direction (Union[Unset, None, SortDirection]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['IncidentModel']]
    """

    kwargs = _get_kwargs(
        connection_name=connection_name,
        months=months,
        open_=open_,
        acknowledged=acknowledged,
        resolved=resolved,
        muted=muted,
        severity=severity,
        page=page,
        limit=limit,
        filter_=filter_,
        dimension=dimension,
        category=category,
        order=order,
        direction=direction,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    connection_name: str,
    *,
    client: AuthenticatedClient,
    months: Union[Unset, None, int] = UNSET,
    open_: Union[Unset, None, bool] = UNSET,
    acknowledged: Union[Unset, None, bool] = UNSET,
    resolved: Union[Unset, None, bool] = UNSET,
    muted: Union[Unset, None, bool] = UNSET,
    severity: Union[Unset, None, int] = UNSET,
    page: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    filter_: Union[Unset, None, str] = UNSET,
    dimension: Union[Unset, None, str] = UNSET,
    category: Union[Unset, None, str] = UNSET,
    order: Union[Unset, None, IncidentSortOrder] = UNSET,
    direction: Union[Unset, None, SortDirection] = UNSET,
) -> Optional[List["IncidentModel"]]:
    """findRecentIncidentsOnConnection

     Returns a list of recent data quality incidents.

    Args:
        connection_name (str):
        months (Union[Unset, None, int]):
        open_ (Union[Unset, None, bool]):
        acknowledged (Union[Unset, None, bool]):
        resolved (Union[Unset, None, bool]):
        muted (Union[Unset, None, bool]):
        severity (Union[Unset, None, int]):
        page (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        filter_ (Union[Unset, None, str]):
        dimension (Union[Unset, None, str]):
        category (Union[Unset, None, str]):
        order (Union[Unset, None, IncidentSortOrder]):
        direction (Union[Unset, None, SortDirection]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['IncidentModel']
    """

    return (
        await asyncio_detailed(
            connection_name=connection_name,
            client=client,
            months=months,
            open_=open_,
            acknowledged=acknowledged,
            resolved=resolved,
            muted=muted,
            severity=severity,
            page=page,
            limit=limit,
            filter_=filter_,
            dimension=dimension,
            category=category,
            order=order,
            direction=direction,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_anomaly_daily_monitoring_checks_spec_custom_checks import (
        ColumnAnomalyDailyMonitoringChecksSpecCustomChecks,
    )
    from ..models.column_max_anomaly_differencing_check_spec import (
        ColumnMaxAnomalyDifferencingCheckSpec,
    )
    from ..models.column_mean_anomaly_stationary_check_spec import (
        ColumnMeanAnomalyStationaryCheckSpec,
    )
    from ..models.column_mean_change_1_day_check_spec import (
        ColumnMeanChange1DayCheckSpec,
    )
    from ..models.column_mean_change_7_days_check_spec import (
        ColumnMeanChange7DaysCheckSpec,
    )
    from ..models.column_mean_change_30_days_check_spec import (
        ColumnMeanChange30DaysCheckSpec,
    )
    from ..models.column_mean_change_check_spec import ColumnMeanChangeCheckSpec
    from ..models.column_median_anomaly_stationary_check_spec import (
        ColumnMedianAnomalyStationaryCheckSpec,
    )
    from ..models.column_median_change_1_day_check_spec import (
        ColumnMedianChange1DayCheckSpec,
    )
    from ..models.column_median_change_7_days_check_spec import (
        ColumnMedianChange7DaysCheckSpec,
    )
    from ..models.column_median_change_30_days_check_spec import (
        ColumnMedianChange30DaysCheckSpec,
    )
    from ..models.column_median_change_check_spec import ColumnMedianChangeCheckSpec
    from ..models.column_min_anomaly_differencing_check_spec import (
        ColumnMinAnomalyDifferencingCheckSpec,
    )
    from ..models.column_sum_anomaly_differencing_check_spec import (
        ColumnSumAnomalyDifferencingCheckSpec,
    )
    from ..models.column_sum_change_1_day_check_spec import ColumnSumChange1DayCheckSpec
    from ..models.column_sum_change_7_days_check_spec import (
        ColumnSumChange7DaysCheckSpec,
    )
    from ..models.column_sum_change_30_days_check_spec import (
        ColumnSumChange30DaysCheckSpec,
    )
    from ..models.column_sum_change_check_spec import ColumnSumChangeCheckSpec


T = TypeVar("T", bound="ColumnAnomalyDailyMonitoringChecksSpec")


@_attrs_define
class ColumnAnomalyDailyMonitoringChecksSpec:
    """
    Attributes:
        custom_checks (Union[Unset, ColumnAnomalyDailyMonitoringChecksSpecCustomChecks]): Dictionary of additional
            custom checks within this category. The keys are check names defined in the definition section. The sensor
            parameters and rules should match the type of the configured sensor and rule for the custom check.
        daily_sum_anomaly (Union[Unset, ColumnSumAnomalyDifferencingCheckSpec]):
        daily_mean_anomaly (Union[Unset, ColumnMeanAnomalyStationaryCheckSpec]):
        daily_median_anomaly (Union[Unset, ColumnMedianAnomalyStationaryCheckSpec]):
        daily_min_anomaly (Union[Unset, ColumnMinAnomalyDifferencingCheckSpec]):
        daily_max_anomaly (Union[Unset, ColumnMaxAnomalyDifferencingCheckSpec]):
        daily_mean_change (Union[Unset, ColumnMeanChangeCheckSpec]):
        daily_median_change (Union[Unset, ColumnMedianChangeCheckSpec]):
        daily_sum_change (Union[Unset, ColumnSumChangeCheckSpec]):
        daily_mean_change_1_day (Union[Unset, ColumnMeanChange1DayCheckSpec]):
        daily_mean_change_7_days (Union[Unset, ColumnMeanChange7DaysCheckSpec]):
        daily_mean_change_30_days (Union[Unset, ColumnMeanChange30DaysCheckSpec]):
        daily_median_change_1_day (Union[Unset, ColumnMedianChange1DayCheckSpec]):
        daily_median_change_7_days (Union[Unset, ColumnMedianChange7DaysCheckSpec]):
        daily_median_change_30_days (Union[Unset, ColumnMedianChange30DaysCheckSpec]):
        daily_sum_change_1_day (Union[Unset, ColumnSumChange1DayCheckSpec]):
        daily_sum_change_7_days (Union[Unset, ColumnSumChange7DaysCheckSpec]):
        daily_sum_change_30_days (Union[Unset, ColumnSumChange30DaysCheckSpec]):
    """

    custom_checks: Union[
        Unset, "ColumnAnomalyDailyMonitoringChecksSpecCustomChecks"
    ] = UNSET
    daily_sum_anomaly: Union[Unset, "ColumnSumAnomalyDifferencingCheckSpec"] = UNSET
    daily_mean_anomaly: Union[Unset, "ColumnMeanAnomalyStationaryCheckSpec"] = UNSET
    daily_median_anomaly: Union[Unset, "ColumnMedianAnomalyStationaryCheckSpec"] = UNSET
    daily_min_anomaly: Union[Unset, "ColumnMinAnomalyDifferencingCheckSpec"] = UNSET
    daily_max_anomaly: Union[Unset, "ColumnMaxAnomalyDifferencingCheckSpec"] = UNSET
    daily_mean_change: Union[Unset, "ColumnMeanChangeCheckSpec"] = UNSET
    daily_median_change: Union[Unset, "ColumnMedianChangeCheckSpec"] = UNSET
    daily_sum_change: Union[Unset, "ColumnSumChangeCheckSpec"] = UNSET
    daily_mean_change_1_day: Union[Unset, "ColumnMeanChange1DayCheckSpec"] = UNSET
    daily_mean_change_7_days: Union[Unset, "ColumnMeanChange7DaysCheckSpec"] = UNSET
    daily_mean_change_30_days: Union[Unset, "ColumnMeanChange30DaysCheckSpec"] = UNSET
    daily_median_change_1_day: Union[Unset, "ColumnMedianChange1DayCheckSpec"] = UNSET
    daily_median_change_7_days: Union[Unset, "ColumnMedianChange7DaysCheckSpec"] = UNSET
    daily_median_change_30_days: Union[Unset, "ColumnMedianChange30DaysCheckSpec"] = (
        UNSET
    )
    daily_sum_change_1_day: Union[Unset, "ColumnSumChange1DayCheckSpec"] = UNSET
    daily_sum_change_7_days: Union[Unset, "ColumnSumChange7DaysCheckSpec"] = UNSET
    daily_sum_change_30_days: Union[Unset, "ColumnSumChange30DaysCheckSpec"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        custom_checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_checks, Unset):
            custom_checks = self.custom_checks.to_dict()

        daily_sum_anomaly: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_sum_anomaly, Unset):
            daily_sum_anomaly = self.daily_sum_anomaly.to_dict()

        daily_mean_anomaly: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_mean_anomaly, Unset):
            daily_mean_anomaly = self.daily_mean_anomaly.to_dict()

        daily_median_anomaly: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_median_anomaly, Unset):
            daily_median_anomaly = self.daily_median_anomaly.to_dict()

        daily_min_anomaly: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_min_anomaly, Unset):
            daily_min_anomaly = self.daily_min_anomaly.to_dict()

        daily_max_anomaly: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_max_anomaly, Unset):
            daily_max_anomaly = self.daily_max_anomaly.to_dict()

        daily_mean_change: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_mean_change, Unset):
            daily_mean_change = self.daily_mean_change.to_dict()

        daily_median_change: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_median_change, Unset):
            daily_median_change = self.daily_median_change.to_dict()

        daily_sum_change: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_sum_change, Unset):
            daily_sum_change = self.daily_sum_change.to_dict()

        daily_mean_change_1_day: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_mean_change_1_day, Unset):
            daily_mean_change_1_day = self.daily_mean_change_1_day.to_dict()

        daily_mean_change_7_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_mean_change_7_days, Unset):
            daily_mean_change_7_days = self.daily_mean_change_7_days.to_dict()

        daily_mean_change_30_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_mean_change_30_days, Unset):
            daily_mean_change_30_days = self.daily_mean_change_30_days.to_dict()

        daily_median_change_1_day: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_median_change_1_day, Unset):
            daily_median_change_1_day = self.daily_median_change_1_day.to_dict()

        daily_median_change_7_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_median_change_7_days, Unset):
            daily_median_change_7_days = self.daily_median_change_7_days.to_dict()

        daily_median_change_30_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_median_change_30_days, Unset):
            daily_median_change_30_days = self.daily_median_change_30_days.to_dict()

        daily_sum_change_1_day: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_sum_change_1_day, Unset):
            daily_sum_change_1_day = self.daily_sum_change_1_day.to_dict()

        daily_sum_change_7_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_sum_change_7_days, Unset):
            daily_sum_change_7_days = self.daily_sum_change_7_days.to_dict()

        daily_sum_change_30_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_sum_change_30_days, Unset):
            daily_sum_change_30_days = self.daily_sum_change_30_days.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if custom_checks is not UNSET:
            field_dict["custom_checks"] = custom_checks
        if daily_sum_anomaly is not UNSET:
            field_dict["daily_sum_anomaly"] = daily_sum_anomaly
        if daily_mean_anomaly is not UNSET:
            field_dict["daily_mean_anomaly"] = daily_mean_anomaly
        if daily_median_anomaly is not UNSET:
            field_dict["daily_median_anomaly"] = daily_median_anomaly
        if daily_min_anomaly is not UNSET:
            field_dict["daily_min_anomaly"] = daily_min_anomaly
        if daily_max_anomaly is not UNSET:
            field_dict["daily_max_anomaly"] = daily_max_anomaly
        if daily_mean_change is not UNSET:
            field_dict["daily_mean_change"] = daily_mean_change
        if daily_median_change is not UNSET:
            field_dict["daily_median_change"] = daily_median_change
        if daily_sum_change is not UNSET:
            field_dict["daily_sum_change"] = daily_sum_change
        if daily_mean_change_1_day is not UNSET:
            field_dict["daily_mean_change_1_day"] = daily_mean_change_1_day
        if daily_mean_change_7_days is not UNSET:
            field_dict["daily_mean_change_7_days"] = daily_mean_change_7_days
        if daily_mean_change_30_days is not UNSET:
            field_dict["daily_mean_change_30_days"] = daily_mean_change_30_days
        if daily_median_change_1_day is not UNSET:
            field_dict["daily_median_change_1_day"] = daily_median_change_1_day
        if daily_median_change_7_days is not UNSET:
            field_dict["daily_median_change_7_days"] = daily_median_change_7_days
        if daily_median_change_30_days is not UNSET:
            field_dict["daily_median_change_30_days"] = daily_median_change_30_days
        if daily_sum_change_1_day is not UNSET:
            field_dict["daily_sum_change_1_day"] = daily_sum_change_1_day
        if daily_sum_change_7_days is not UNSET:
            field_dict["daily_sum_change_7_days"] = daily_sum_change_7_days
        if daily_sum_change_30_days is not UNSET:
            field_dict["daily_sum_change_30_days"] = daily_sum_change_30_days

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_anomaly_daily_monitoring_checks_spec_custom_checks import (
            ColumnAnomalyDailyMonitoringChecksSpecCustomChecks,
        )
        from ..models.column_max_anomaly_differencing_check_spec import (
            ColumnMaxAnomalyDifferencingCheckSpec,
        )
        from ..models.column_mean_anomaly_stationary_check_spec import (
            ColumnMeanAnomalyStationaryCheckSpec,
        )
        from ..models.column_mean_change_1_day_check_spec import (
            ColumnMeanChange1DayCheckSpec,
        )
        from ..models.column_mean_change_7_days_check_spec import (
            ColumnMeanChange7DaysCheckSpec,
        )
        from ..models.column_mean_change_30_days_check_spec import (
            ColumnMeanChange30DaysCheckSpec,
        )
        from ..models.column_mean_change_check_spec import ColumnMeanChangeCheckSpec
        from ..models.column_median_anomaly_stationary_check_spec import (
            ColumnMedianAnomalyStationaryCheckSpec,
        )
        from ..models.column_median_change_1_day_check_spec import (
            ColumnMedianChange1DayCheckSpec,
        )
        from ..models.column_median_change_7_days_check_spec import (
            ColumnMedianChange7DaysCheckSpec,
        )
        from ..models.column_median_change_30_days_check_spec import (
            ColumnMedianChange30DaysCheckSpec,
        )
        from ..models.column_median_change_check_spec import ColumnMedianChangeCheckSpec
        from ..models.column_min_anomaly_differencing_check_spec import (
            ColumnMinAnomalyDifferencingCheckSpec,
        )
        from ..models.column_sum_anomaly_differencing_check_spec import (
            ColumnSumAnomalyDifferencingCheckSpec,
        )
        from ..models.column_sum_change_1_day_check_spec import (
            ColumnSumChange1DayCheckSpec,
        )
        from ..models.column_sum_change_7_days_check_spec import (
            ColumnSumChange7DaysCheckSpec,
        )
        from ..models.column_sum_change_30_days_check_spec import (
            ColumnSumChange30DaysCheckSpec,
        )
        from ..models.column_sum_change_check_spec import ColumnSumChangeCheckSpec

        d = src_dict.copy()
        _custom_checks = d.pop("custom_checks", UNSET)
        custom_checks: Union[Unset, ColumnAnomalyDailyMonitoringChecksSpecCustomChecks]
        if isinstance(_custom_checks, Unset):
            custom_checks = UNSET
        else:
            custom_checks = (
                ColumnAnomalyDailyMonitoringChecksSpecCustomChecks.from_dict(
                    _custom_checks
                )
            )

        _daily_sum_anomaly = d.pop("daily_sum_anomaly", UNSET)
        daily_sum_anomaly: Union[Unset, ColumnSumAnomalyDifferencingCheckSpec]
        if isinstance(_daily_sum_anomaly, Unset):
            daily_sum_anomaly = UNSET
        else:
            daily_sum_anomaly = ColumnSumAnomalyDifferencingCheckSpec.from_dict(
                _daily_sum_anomaly
            )

        _daily_mean_anomaly = d.pop("daily_mean_anomaly", UNSET)
        daily_mean_anomaly: Union[Unset, ColumnMeanAnomalyStationaryCheckSpec]
        if isinstance(_daily_mean_anomaly, Unset):
            daily_mean_anomaly = UNSET
        else:
            daily_mean_anomaly = ColumnMeanAnomalyStationaryCheckSpec.from_dict(
                _daily_mean_anomaly
            )

        _daily_median_anomaly = d.pop("daily_median_anomaly", UNSET)
        daily_median_anomaly: Union[Unset, ColumnMedianAnomalyStationaryCheckSpec]
        if isinstance(_daily_median_anomaly, Unset):
            daily_median_anomaly = UNSET
        else:
            daily_median_anomaly = ColumnMedianAnomalyStationaryCheckSpec.from_dict(
                _daily_median_anomaly
            )

        _daily_min_anomaly = d.pop("daily_min_anomaly", UNSET)
        daily_min_anomaly: Union[Unset, ColumnMinAnomalyDifferencingCheckSpec]
        if isinstance(_daily_min_anomaly, Unset):
            daily_min_anomaly = UNSET
        else:
            daily_min_anomaly = ColumnMinAnomalyDifferencingCheckSpec.from_dict(
                _daily_min_anomaly
            )

        _daily_max_anomaly = d.pop("daily_max_anomaly", UNSET)
        daily_max_anomaly: Union[Unset, ColumnMaxAnomalyDifferencingCheckSpec]
        if isinstance(_daily_max_anomaly, Unset):
            daily_max_anomaly = UNSET
        else:
            daily_max_anomaly = ColumnMaxAnomalyDifferencingCheckSpec.from_dict(
                _daily_max_anomaly
            )

        _daily_mean_change = d.pop("daily_mean_change", UNSET)
        daily_mean_change: Union[Unset, ColumnMeanChangeCheckSpec]
        if isinstance(_daily_mean_change, Unset):
            daily_mean_change = UNSET
        else:
            daily_mean_change = ColumnMeanChangeCheckSpec.from_dict(_daily_mean_change)

        _daily_median_change = d.pop("daily_median_change", UNSET)
        daily_median_change: Union[Unset, ColumnMedianChangeCheckSpec]
        if isinstance(_daily_median_change, Unset):
            daily_median_change = UNSET
        else:
            daily_median_change = ColumnMedianChangeCheckSpec.from_dict(
                _daily_median_change
            )

        _daily_sum_change = d.pop("daily_sum_change", UNSET)
        daily_sum_change: Union[Unset, ColumnSumChangeCheckSpec]
        if isinstance(_daily_sum_change, Unset):
            daily_sum_change = UNSET
        else:
            daily_sum_change = ColumnSumChangeCheckSpec.from_dict(_daily_sum_change)

        _daily_mean_change_1_day = d.pop("daily_mean_change_1_day", UNSET)
        daily_mean_change_1_day: Union[Unset, ColumnMeanChange1DayCheckSpec]
        if isinstance(_daily_mean_change_1_day, Unset):
            daily_mean_change_1_day = UNSET
        else:
            daily_mean_change_1_day = ColumnMeanChange1DayCheckSpec.from_dict(
                _daily_mean_change_1_day
            )

        _daily_mean_change_7_days = d.pop("daily_mean_change_7_days", UNSET)
        daily_mean_change_7_days: Union[Unset, ColumnMeanChange7DaysCheckSpec]
        if isinstance(_daily_mean_change_7_days, Unset):
            daily_mean_change_7_days = UNSET
        else:
            daily_mean_change_7_days = ColumnMeanChange7DaysCheckSpec.from_dict(
                _daily_mean_change_7_days
            )

        _daily_mean_change_30_days = d.pop("daily_mean_change_30_days", UNSET)
        daily_mean_change_30_days: Union[Unset, ColumnMeanChange30DaysCheckSpec]
        if isinstance(_daily_mean_change_30_days, Unset):
            daily_mean_change_30_days = UNSET
        else:
            daily_mean_change_30_days = ColumnMeanChange30DaysCheckSpec.from_dict(
                _daily_mean_change_30_days
            )

        _daily_median_change_1_day = d.pop("daily_median_change_1_day", UNSET)
        daily_median_change_1_day: Union[Unset, ColumnMedianChange1DayCheckSpec]
        if isinstance(_daily_median_change_1_day, Unset):
            daily_median_change_1_day = UNSET
        else:
            daily_median_change_1_day = ColumnMedianChange1DayCheckSpec.from_dict(
                _daily_median_change_1_day
            )

        _daily_median_change_7_days = d.pop("daily_median_change_7_days", UNSET)
        daily_median_change_7_days: Union[Unset, ColumnMedianChange7DaysCheckSpec]
        if isinstance(_daily_median_change_7_days, Unset):
            daily_median_change_7_days = UNSET
        else:
            daily_median_change_7_days = ColumnMedianChange7DaysCheckSpec.from_dict(
                _daily_median_change_7_days
            )

        _daily_median_change_30_days = d.pop("daily_median_change_30_days", UNSET)
        daily_median_change_30_days: Union[Unset, ColumnMedianChange30DaysCheckSpec]
        if isinstance(_daily_median_change_30_days, Unset):
            daily_median_change_30_days = UNSET
        else:
            daily_median_change_30_days = ColumnMedianChange30DaysCheckSpec.from_dict(
                _daily_median_change_30_days
            )

        _daily_sum_change_1_day = d.pop("daily_sum_change_1_day", UNSET)
        daily_sum_change_1_day: Union[Unset, ColumnSumChange1DayCheckSpec]
        if isinstance(_daily_sum_change_1_day, Unset):
            daily_sum_change_1_day = UNSET
        else:
            daily_sum_change_1_day = ColumnSumChange1DayCheckSpec.from_dict(
                _daily_sum_change_1_day
            )

        _daily_sum_change_7_days = d.pop("daily_sum_change_7_days", UNSET)
        daily_sum_change_7_days: Union[Unset, ColumnSumChange7DaysCheckSpec]
        if isinstance(_daily_sum_change_7_days, Unset):
            daily_sum_change_7_days = UNSET
        else:
            daily_sum_change_7_days = ColumnSumChange7DaysCheckSpec.from_dict(
                _daily_sum_change_7_days
            )

        _daily_sum_change_30_days = d.pop("daily_sum_change_30_days", UNSET)
        daily_sum_change_30_days: Union[Unset, ColumnSumChange30DaysCheckSpec]
        if isinstance(_daily_sum_change_30_days, Unset):
            daily_sum_change_30_days = UNSET
        else:
            daily_sum_change_30_days = ColumnSumChange30DaysCheckSpec.from_dict(
                _daily_sum_change_30_days
            )

        column_anomaly_daily_monitoring_checks_spec = cls(
            custom_checks=custom_checks,
            daily_sum_anomaly=daily_sum_anomaly,
            daily_mean_anomaly=daily_mean_anomaly,
            daily_median_anomaly=daily_median_anomaly,
            daily_min_anomaly=daily_min_anomaly,
            daily_max_anomaly=daily_max_anomaly,
            daily_mean_change=daily_mean_change,
            daily_median_change=daily_median_change,
            daily_sum_change=daily_sum_change,
            daily_mean_change_1_day=daily_mean_change_1_day,
            daily_mean_change_7_days=daily_mean_change_7_days,
            daily_mean_change_30_days=daily_mean_change_30_days,
            daily_median_change_1_day=daily_median_change_1_day,
            daily_median_change_7_days=daily_median_change_7_days,
            daily_median_change_30_days=daily_median_change_30_days,
            daily_sum_change_1_day=daily_sum_change_1_day,
            daily_sum_change_7_days=daily_sum_change_7_days,
            daily_sum_change_30_days=daily_sum_change_30_days,
        )

        column_anomaly_daily_monitoring_checks_spec.additional_properties = d
        return column_anomaly_daily_monitoring_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.issue_histogram_model_checks import IssueHistogramModelChecks
    from ..models.issue_histogram_model_columns import IssueHistogramModelColumns
    from ..models.issue_histogram_model_days import IssueHistogramModelDays


T = TypeVar("T", bound="IssueHistogramModel")


@_attrs_define
class IssueHistogramModel:
    """
    Attributes:
        has_profiling_issues (Union[Unset, bool]): True when this data quality incident is based on data quality issues
            from profiling checks within the filters applied to search for linked data quality issues.
        has_daily_monitoring_issues (Union[Unset, bool]): True when this data quality incident is based on data quality
            issues from daily monitoring checks within the filters applied to search for linked data quality issues.
        has_monthly_monitoring_issues (Union[Unset, bool]): True when this data quality incident is based on data
            quality issues from monthly monitoring checks within the filters applied to search for linked data quality
            issues.
        has_daily_partitioned_issues (Union[Unset, bool]): True when this data quality incident is based on data quality
            issues from daily partitioned checks within the filters applied to search for linked data quality issues.
        has_monthly_partitioned_issues (Union[Unset, bool]): True when this data quality incident is based on data
            quality issues from monthly partitioned checks within the filters applied to search for linked data quality
            issues.
        days (Union[Unset, IssueHistogramModelDays]): A map of the numbers of data quality issues per day, the day uses
            the DQOps server timezone.
        columns (Union[Unset, IssueHistogramModelColumns]): A map of column names with the most data quality issues
            related to the incident. The map returns the count of issues as the value.
        checks (Union[Unset, IssueHistogramModelChecks]): A map of data quality check names with the most data quality
            issues related to the incident. The map returns the count of issues as the value.
    """

    has_profiling_issues: Union[Unset, bool] = UNSET
    has_daily_monitoring_issues: Union[Unset, bool] = UNSET
    has_monthly_monitoring_issues: Union[Unset, bool] = UNSET
    has_daily_partitioned_issues: Union[Unset, bool] = UNSET
    has_monthly_partitioned_issues: Union[Unset, bool] = UNSET
    days: Union[Unset, "IssueHistogramModelDays"] = UNSET
    columns: Union[Unset, "IssueHistogramModelColumns"] = UNSET
    checks: Union[Unset, "IssueHistogramModelChecks"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        has_profiling_issues = self.has_profiling_issues
        has_daily_monitoring_issues = self.has_daily_monitoring_issues
        has_monthly_monitoring_issues = self.has_monthly_monitoring_issues
        has_daily_partitioned_issues = self.has_daily_partitioned_issues
        has_monthly_partitioned_issues = self.has_monthly_partitioned_issues
        days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.days, Unset):
            days = self.days.to_dict()

        columns: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.columns, Unset):
            columns = self.columns.to_dict()

        checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.checks, Unset):
            checks = self.checks.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if has_profiling_issues is not UNSET:
            field_dict["hasProfilingIssues"] = has_profiling_issues
        if has_daily_monitoring_issues is not UNSET:
            field_dict["hasDailyMonitoringIssues"] = has_daily_monitoring_issues
        if has_monthly_monitoring_issues is not UNSET:
            field_dict["hasMonthlyMonitoringIssues"] = has_monthly_monitoring_issues
        if has_daily_partitioned_issues is not UNSET:
            field_dict["hasDailyPartitionedIssues"] = has_daily_partitioned_issues
        if has_monthly_partitioned_issues is not UNSET:
            field_dict["hasMonthlyPartitionedIssues"] = has_monthly_partitioned_issues
        if days is not UNSET:
            field_dict["days"] = days
        if columns is not UNSET:
            field_dict["columns"] = columns
        if checks is not UNSET:
            field_dict["checks"] = checks

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.issue_histogram_model_checks import IssueHistogramModelChecks
        from ..models.issue_histogram_model_columns import IssueHistogramModelColumns
        from ..models.issue_histogram_model_days import IssueHistogramModelDays

        d = src_dict.copy()
        has_profiling_issues = d.pop("hasProfilingIssues", UNSET)

        has_daily_monitoring_issues = d.pop("hasDailyMonitoringIssues", UNSET)

        has_monthly_monitoring_issues = d.pop("hasMonthlyMonitoringIssues", UNSET)

        has_daily_partitioned_issues = d.pop("hasDailyPartitionedIssues", UNSET)

        has_monthly_partitioned_issues = d.pop("hasMonthlyPartitionedIssues", UNSET)

        _days = d.pop("days", UNSET)
        days: Union[Unset, IssueHistogramModelDays]
        if isinstance(_days, Unset):
            days = UNSET
        else:
            days = IssueHistogramModelDays.from_dict(_days)

        _columns = d.pop("columns", UNSET)
        columns: Union[Unset, IssueHistogramModelColumns]
        if isinstance(_columns, Unset):
            columns = UNSET
        else:
            columns = IssueHistogramModelColumns.from_dict(_columns)

        _checks = d.pop("checks", UNSET)
        checks: Union[Unset, IssueHistogramModelChecks]
        if isinstance(_checks, Unset):
            checks = UNSET
        else:
            checks = IssueHistogramModelChecks.from_dict(_checks)

        issue_histogram_model = cls(
            has_profiling_issues=has_profiling_issues,
            has_daily_monitoring_issues=has_daily_monitoring_issues,
            has_monthly_monitoring_issues=has_monthly_monitoring_issues,
            has_daily_partitioned_issues=has_daily_partitioned_issues,
            has_monthly_partitioned_issues=has_monthly_partitioned_issues,
            days=days,
            columns=columns,
            checks=checks,
        )

        issue_histogram_model.additional_properties = d
        return issue_histogram_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.issue_histogram_model import IssueHistogramModel
from ...types import UNSET, Response, Unset


def _get_kwargs(
    connection_name: str,
    year: int,
    month: int,
    incident_id: str,
    *,
    filter_: Union[Unset, None, str] = UNSET,
    days: Union[Unset, None, int] = UNSET,
    date: Union[Unset, None, datetime.date] = UNSET,
    column: Union[Unset, None, str] = UNSET,
    check: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:

    pass

    params: Dict[str, Any] = {}
    params["filter"] = filter_

    params["days"] = days

    json_date: Union[Unset, None, str] = UNSET
    if not isinstance(date, Unset):
        json_date = date.isoformat() if date else None

    params["date"] = json_date

    params["column"] = column

    params["check"] = check

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "api/incidents/{connectionName}/{year}/{month}/{incidentId}/histogram".format(
            connectionName=connection_name,
            year=year,
            month=month,
            incidentId=incident_id,
        ),
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[IssueHistogramModel]:
    if response.status_code == HTTPStatus.OK:
        response_200 = IssueHistogramModel.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[IssueHistogramModel]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    connection_name: str,
    year: int,
    month: int,
    incident_id: str,
    *,
    client: AuthenticatedClient,
    filter_: Union[Unset, None, str] = UNSET,
    days: Union[Unset, None, int] = UNSET,
    date: Union[Unset, None, datetime.date] = UNSET,
    column: Union[Unset, None, str] = UNSET,
    check: Union[Unset, None, str] = UNSET,
) -> Response[IssueHistogramModel]:
    """getIncidentHistogram

     Generates a histogram of data quality issues for each day, returning the number of data quality
    issues on that day. The other histograms are by a column name and by a check name.

    Args:
        connection_name (str):
        year (int):
        month (int):
        incident_id (str):
        filter_ (Union[Unset, None, str]):
        days (Union[Unset, None, int]):
        date (Union[Unset, None, datetime.date]):
        column (Union[Unset, None, str]):
        check (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[IssueHistogramModel]
    """

    kwargs = _get_kwargs(
        connection_name=connection_name,
        year=year,
        month=month,
        incident_id=incident_id,
        filter_=filter_,
        days=days,
        date=date,
        column=column,
        check=check,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    connection_name: str,
    year: int,
    month: int,
    incident_id: str,
    *,
    client: AuthenticatedClient,
    filter_: Union[Unset, None, str] = UNSET,
    days: Union[Unset, None, int] = UNSET,
    date: Union[Unset, None, datetime.date] = UNSET,
    column: Union[Unset, None, str] = UNSET,
    check: Union[Unset, None, str] = UNSET,
) -> Optional[IssueHistogramModel]:
    """getIncidentHistogram

     Generates a histogram of data quality issues for each day, returning the number of data quality
    issues on that day. The other histograms are by a column name and by a check name.

    Args:
        connection_name (str):
        year (int):
        month (int):
        incident_id (str):
        filter_ (Union[Unset, None, str]):
        days (Union[Unset, None, int]):
        date (Union[Unset, None, datetime.date]):
        column (Union[Unset, None, str]):
        check (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        IssueHistogramModel
    """

    return sync_detailed(
        connection_name=connection_name,
        year=year,
        month=month,
        incident_id=incident_id,
        client=client,
        filter_=filter_,
        days=days,
        date=date,
        column=column,
        check=check,
    ).parsed


async def asyncio_detailed(
    connection_name: str,
    year: int,
    month: int,
    incident_id: str,
    *,
    client: AuthenticatedClient,
    filter_: Union[Unset, None, str] = UNSET,
    days: Union[Unset, None, int] = UNSET,
    date: Union[Unset, None, datetime.date] = UNSET,
    column: Union[Unset, None, str] = UNSET,
    check: Union[Unset, None, str] = UNSET,
) -> Response[IssueHistogramModel]:
    """getIncidentHistogram

     Generates a histogram of data quality issues for each day, returning the number of data quality
    issues on that day. The other histograms are by a column name and by a check name.

    Args:
        connection_name (str):
        year (int):
        month (int):
        incident_id (str):
        filter_ (Union[Unset, None, str]):
        days (Union[Unset, None, int]):
        date (Union[Unset, None, datetime.date]):
        column (Union[Unset, None, str]):
        check (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[IssueHistogramModel]
    """

    kwargs = _get_kwargs(
        connection_name=connection_name,
        year=year,
        month=month,
        incident_id=incident_id,
        filter_=filter_,
        days=days,
        date=date,
        column=column,
        check=check,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    connection_name: str,
    year: int,
    month: int,
    incident_id: str,
    *,
    client: AuthenticatedClient,
    filter_: Union[Unset, None, str] = UNSET,
    days: Union[Unset, None, int] = UNSET,
    date: Union[Unset, None, datetime.date] = UNSET,
    column: Union[Unset, None, str] = UNSET,
    check: Union[Unset, None, str] = UNSET,
) -> Optional[IssueHistogramModel]:
    """getIncidentHistogram

     Generates a histogram of data quality issues for each day, returning the number of data quality
    issues on that day. The other histograms are by a column name and by a check name.

    Args:
        connection_name (str):
        year (int):
        month (int):
        incident_id (str):
        filter_ (Union[Unset, None, str]):
        days (Union[Unset, None, int]):
        date (Union[Unset, None, datetime.date]):
        column (Union[Unset, None, str]):
        check (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        IssueHistogramModel
    """

    return (
        await asyncio_detailed(
            connection_name=connection_name,
            year=year,
            month=month,
            incident_id=incident_id,
            client=client,
            filter_=filter_,
            days=days,
            date=date,
            column=column,
            check=check,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dashboard_spec import DashboardSpec


T = TypeVar("T", bound="DashboardsFolderSpec")


@_attrs_define
class DashboardsFolderSpec:
    """
    Attributes:
        folder_name (Union[Unset, str]): Folder name
        standard (Union[Unset, bool]): Always shows this schema tree node because it contains standard dashboards. Set
            the value to false to show this folder only when advanced dashboards are enabled.
        dashboards (Union[Unset, List['DashboardSpec']]): List of data quality dashboard at this level.
        folders (Union[Unset, List['DashboardsFolderSpec']]): List of data quality dashboard folders at this level.
        hide_folder (Union[Unset, bool]): Hides the whole folder and all nested dashboards from the navigation tree. If
            you want to hide some of the build-in folders, update the settings/dashboardslist.dqodashboards.yaml file in the
            DQOps user home folder, create an empty folder with the same name as a built-in folder, and set the value of
            this field to true.
    """

    folder_name: Union[Unset, str] = UNSET
    standard: Union[Unset, bool] = UNSET
    dashboards: Union[Unset, List["DashboardSpec"]] = UNSET
    folders: Union[Unset, List["DashboardsFolderSpec"]] = UNSET
    hide_folder: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        folder_name = self.folder_name
        standard = self.standard
        dashboards: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.dashboards, Unset):
            dashboards = []
            for dashboards_item_data in self.dashboards:
                dashboards_item = dashboards_item_data.to_dict()

                dashboards.append(dashboards_item)

        folders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.folders, Unset):
            folders = []
            for folders_item_data in self.folders:
                folders_item = folders_item_data.to_dict()

                folders.append(folders_item)

        hide_folder = self.hide_folder

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if folder_name is not UNSET:
            field_dict["folder_name"] = folder_name
        if standard is not UNSET:
            field_dict["standard"] = standard
        if dashboards is not UNSET:
            field_dict["dashboards"] = dashboards
        if folders is not UNSET:
            field_dict["folders"] = folders
        if hide_folder is not UNSET:
            field_dict["hide_folder"] = hide_folder

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.dashboard_spec import DashboardSpec

        d = src_dict.copy()
        folder_name = d.pop("folder_name", UNSET)

        standard = d.pop("standard", UNSET)

        dashboards = []
        _dashboards = d.pop("dashboards", UNSET)
        for dashboards_item_data in _dashboards or []:
            dashboards_item = DashboardSpec.from_dict(dashboards_item_data)

            dashboards.append(dashboards_item)

        folders = []
        _folders = d.pop("folders", UNSET)
        for folders_item_data in _folders or []:
            folders_item = DashboardsFolderSpec.from_dict(folders_item_data)

            folders.append(folders_item)

        hide_folder = d.pop("hide_folder", UNSET)

        dashboards_folder_spec = cls(
            folder_name=folder_name,
            standard=standard,
            dashboards=dashboards,
            folders=folders,
            hide_folder=hide_folder,
        )

        dashboards_folder_spec.additional_properties = d
        return dashboards_folder_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ImportTablesQueueJobParameters")


@_attrs_define
class ImportTablesQueueJobParameters:
    """
    Attributes:
        connection_name (Union[Unset, str]): Connection name
        schema_name (Union[Unset, str]): Schema name
        table_name_contains (Union[Unset, str]): Optional filter for the table names to import. The table names that are
            imported must contain a substring matching this parameter. This filter is case sensitive.
        table_names (Union[Unset, List[str]]): Optional list of table names inside the schema. When the list of tables
            is empty, all tables are imported.
        tables_import_limit (Union[Unset, int]): Optional parameter to configure the limit of tables that are imported
            from a single schema. Leave this parameter blank to use the default limit (300 tables).
    """

    connection_name: Union[Unset, str] = UNSET
    schema_name: Union[Unset, str] = UNSET
    table_name_contains: Union[Unset, str] = UNSET
    table_names: Union[Unset, List[str]] = UNSET
    tables_import_limit: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connection_name = self.connection_name
        schema_name = self.schema_name
        table_name_contains = self.table_name_contains
        table_names: Union[Unset, List[str]] = UNSET
        if not isinstance(self.table_names, Unset):
            table_names = self.table_names

        tables_import_limit = self.tables_import_limit

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connection_name is not UNSET:
            field_dict["connectionName"] = connection_name
        if schema_name is not UNSET:
            field_dict["schemaName"] = schema_name
        if table_name_contains is not UNSET:
            field_dict["tableNameContains"] = table_name_contains
        if table_names is not UNSET:
            field_dict["tableNames"] = table_names
        if tables_import_limit is not UNSET:
            field_dict["tablesImportLimit"] = tables_import_limit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        connection_name = d.pop("connectionName", UNSET)

        schema_name = d.pop("schemaName", UNSET)

        table_name_contains = d.pop("tableNameContains", UNSET)

        table_names = cast(List[str], d.pop("tableNames", UNSET))

        tables_import_limit = d.pop("tablesImportLimit", UNSET)

        import_tables_queue_job_parameters = cls(
            connection_name=connection_name,
            schema_name=schema_name,
            table_name_contains=table_name_contains,
            table_names=table_names,
            tables_import_limit=tables_import_limit,
        )

        import_tables_queue_job_parameters.additional_properties = d
        return import_tables_queue_job_parameters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.check_current_data_quality_status_model import (
        CheckCurrentDataQualityStatusModel,
    )


T = TypeVar("T", bound="ColumnCurrentDataQualityStatusModelChecks")


@_attrs_define
class ColumnCurrentDataQualityStatusModelChecks:
    """The dictionary of statuses for data quality checks. The keys are data quality check names, the values are the
    current data quality check statuses that describe the most current status.

    """

    additional_properties: Dict[str, "CheckCurrentDataQualityStatusModel"] = (
        _attrs_field(init=False, factory=dict)
    )

    def to_dict(self) -> Dict[str, Any]:
        pass

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()

        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.check_current_data_quality_status_model import (
            CheckCurrentDataQualityStatusModel,
        )

        d = src_dict.copy()
        column_current_data_quality_status_model_checks = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = CheckCurrentDataQualityStatusModel.from_dict(
                prop_dict
            )

            additional_properties[prop_name] = additional_property

        column_current_data_quality_status_model_checks.additional_properties = (
            additional_properties
        )
        return column_current_data_quality_status_model_checks

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> "CheckCurrentDataQualityStatusModel":
        return self.additional_properties[key]

    def __setitem__(
        self, key: str, value: "CheckCurrentDataQualityStatusModel"
    ) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.check_time_scale import CheckTimeScale
from ..models.check_type import CheckType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.physical_table_name import PhysicalTableName
    from ..models.table_comparison_grouping_column_pair_model import (
        TableComparisonGroupingColumnPairModel,
    )


T = TypeVar("T", bound="TableComparisonConfigurationModel")


@_attrs_define
class TableComparisonConfigurationModel:
    """Model that contains the basic information about a table comparison configuration that specifies how the current
    table can be compared with another table that is a source of truth for comparison.

        Attributes:
            table_comparison_configuration_name (Union[Unset, str]): The name of the table comparison configuration that is
                defined in the 'table_comparisons' node on the table specification.
            compared_connection (Union[Unset, str]): Compared connection name - the connection name to the data source that
                is compared (verified).
            compared_table (Union[Unset, PhysicalTableName]):
            reference_connection (Union[Unset, str]): Reference connection name - the connection name to the data source
                that has the reference data to compare to.
            reference_table (Union[Unset, PhysicalTableName]):
            check_type (Union[Unset, CheckType]):
            time_scale (Union[Unset, CheckTimeScale]):
            compared_table_filter (Union[Unset, str]): Optional custom SQL filter expression that is added to the SQL query
                that retrieves the data from the compared table. This expression must be a SQL expression that will be added to
                the WHERE clause when querying the compared table.
            reference_table_filter (Union[Unset, str]): Optional custom SQL filter expression that is added to the SQL query
                that retrieves the data from the reference table (the source of truth). This expression must be a SQL expression
                that will be added to the WHERE clause when querying the reference table.
            grouping_columns (Union[Unset, List['TableComparisonGroupingColumnPairModel']]): List of column pairs from both
                the compared table and the reference table that are used in a GROUP BY clause  for grouping both the compared
                table and the reference table (the source of truth). The columns are used in the next of the table comparison to
                join the results of data groups (row counts, sums of columns) between the compared table and the reference table
                to compare the differences.
            can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can update or delete the table
                comparison.
            can_run_compare_checks (Union[Unset, bool]): Boolean flag that decides if the current user can run comparison
                checks.
            can_delete_data (Union[Unset, bool]): Boolean flag that decides if the current user can delete data (results).
    """

    table_comparison_configuration_name: Union[Unset, str] = UNSET
    compared_connection: Union[Unset, str] = UNSET
    compared_table: Union[Unset, "PhysicalTableName"] = UNSET
    reference_connection: Union[Unset, str] = UNSET
    reference_table: Union[Unset, "PhysicalTableName"] = UNSET
    check_type: Union[Unset, CheckType] = UNSET
    time_scale: Union[Unset, CheckTimeScale] = UNSET
    compared_table_filter: Union[Unset, str] = UNSET
    reference_table_filter: Union[Unset, str] = UNSET
    grouping_columns: Union[Unset, List["TableComparisonGroupingColumnPairModel"]] = (
        UNSET
    )
    can_edit: Union[Unset, bool] = UNSET
    can_run_compare_checks: Union[Unset, bool] = UNSET
    can_delete_data: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        table_comparison_configuration_name = self.table_comparison_configuration_name
        compared_connection = self.compared_connection
        compared_table: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.compared_table, Unset):
            compared_table = self.compared_table.to_dict()

        reference_connection = self.reference_connection
        reference_table: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.reference_table, Unset):
            reference_table = self.reference_table.to_dict()

        check_type: Union[Unset, str] = UNSET
        if not isinstance(self.check_type, Unset):
            check_type = self.check_type.value

        time_scale: Union[Unset, str] = UNSET
        if not isinstance(self.time_scale, Unset):
            time_scale = self.time_scale.value

        compared_table_filter = self.compared_table_filter
        reference_table_filter = self.reference_table_filter
        grouping_columns: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.grouping_columns, Unset):
            grouping_columns = []
            for grouping_columns_item_data in self.grouping_columns:
                grouping_columns_item = grouping_columns_item_data.to_dict()

                grouping_columns.append(grouping_columns_item)

        can_edit = self.can_edit
        can_run_compare_checks = self.can_run_compare_checks
        can_delete_data = self.can_delete_data

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if table_comparison_configuration_name is not UNSET:
            field_dict["table_comparison_configuration_name"] = (
                table_comparison_configuration_name
            )
        if compared_connection is not UNSET:
            field_dict["compared_connection"] = compared_connection
        if compared_table is not UNSET:
            field_dict["compared_table"] = compared_table
        if reference_connection is not UNSET:
            field_dict["reference_connection"] = reference_connection
        if reference_table is not UNSET:
            field_dict["reference_table"] = reference_table
        if check_type is not UNSET:
            field_dict["check_type"] = check_type
        if time_scale is not UNSET:
            field_dict["time_scale"] = time_scale
        if compared_table_filter is not UNSET:
            field_dict["compared_table_filter"] = compared_table_filter
        if reference_table_filter is not UNSET:
            field_dict["reference_table_filter"] = reference_table_filter
        if grouping_columns is not UNSET:
            field_dict["grouping_columns"] = grouping_columns
        if can_edit is not UNSET:
            field_dict["can_edit"] = can_edit
        if can_run_compare_checks is not UNSET:
            field_dict["can_run_compare_checks"] = can_run_compare_checks
        if can_delete_data is not UNSET:
            field_dict["can_delete_data"] = can_delete_data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.physical_table_name import PhysicalTableName
        from ..models.table_comparison_grouping_column_pair_model import (
            TableComparisonGroupingColumnPairModel,
        )

        d = src_dict.copy()
        table_comparison_configuration_name = d.pop(
            "table_comparison_configuration_name", UNSET
        )

        compared_connection = d.pop("compared_connection", UNSET)

        _compared_table = d.pop("compared_table", UNSET)
        compared_table: Union[Unset, PhysicalTableName]
        if isinstance(_compared_table, Unset):
            compared_table = UNSET
        else:
            compared_table = PhysicalTableName.from_dict(_compared_table)

        reference_connection = d.pop("reference_connection", UNSET)

        _reference_table = d.pop("reference_table", UNSET)
        reference_table: Union[Unset, PhysicalTableName]
        if isinstance(_reference_table, Unset):
            reference_table = UNSET
        else:
            reference_table = PhysicalTableName.from_dict(_reference_table)

        _check_type = d.pop("check_type", UNSET)
        check_type: Union[Unset, CheckType]
        if isinstance(_check_type, Unset):
            check_type = UNSET
        else:
            check_type = CheckType(_check_type)

        _time_scale = d.pop("time_scale", UNSET)
        time_scale: Union[Unset, CheckTimeScale]
        if isinstance(_time_scale, Unset):
            time_scale = UNSET
        else:
            time_scale = CheckTimeScale(_time_scale)

        compared_table_filter = d.pop("compared_table_filter", UNSET)

        reference_table_filter = d.pop("reference_table_filter", UNSET)

        grouping_columns = []
        _grouping_columns = d.pop("grouping_columns", UNSET)
        for grouping_columns_item_data in _grouping_columns or []:
            grouping_columns_item = TableComparisonGroupingColumnPairModel.from_dict(
                grouping_columns_item_data
            )

            grouping_columns.append(grouping_columns_item)

        can_edit = d.pop("can_edit", UNSET)

        can_run_compare_checks = d.pop("can_run_compare_checks", UNSET)

        can_delete_data = d.pop("can_delete_data", UNSET)

        table_comparison_configuration_model = cls(
            table_comparison_configuration_name=table_comparison_configuration_name,
            compared_connection=compared_connection,
            compared_table=compared_table,
            reference_connection=reference_connection,
            reference_table=reference_table,
            check_type=check_type,
            time_scale=time_scale,
            compared_table_filter=compared_table_filter,
            reference_table_filter=reference_table_filter,
            grouping_columns=grouping_columns,
            can_edit=can_edit,
            can_run_compare_checks=can_run_compare_checks,
            can_delete_data=can_delete_data,
        )

        table_comparison_configuration_model.additional_properties = d
        return table_comparison_configuration_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CollectStatisticsResult")


@_attrs_define
class CollectStatisticsResult:
    """Returns the result with the summary of the statistics collected.

    Attributes:
        executed_statistics_collectors (Union[Unset, int]): The total count of all executed statistics collectors.
        columns_analyzed (Union[Unset, int]): The count of columns for which DQOps executed a collector and tried to
            read the statistics.
        columns_successfully_analyzed (Union[Unset, int]): The count of columns for which DQOps managed to obtain
            statistics.
        total_collectors_failed (Union[Unset, int]): The count of statistics collectors that failed to execute.
        total_collected_results (Union[Unset, int]): The total number of results that were collected.
    """

    executed_statistics_collectors: Union[Unset, int] = UNSET
    columns_analyzed: Union[Unset, int] = UNSET
    columns_successfully_analyzed: Union[Unset, int] = UNSET
    total_collectors_failed: Union[Unset, int] = UNSET
    total_collected_results: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        executed_statistics_collectors = self.executed_statistics_collectors
        columns_analyzed = self.columns_analyzed
        columns_successfully_analyzed = self.columns_successfully_analyzed
        total_collectors_failed = self.total_collectors_failed
        total_collected_results = self.total_collected_results

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if executed_statistics_collectors is not UNSET:
            field_dict["executed_statistics_collectors"] = (
                executed_statistics_collectors
            )
        if columns_analyzed is not UNSET:
            field_dict["columns_analyzed"] = columns_analyzed
        if columns_successfully_analyzed is not UNSET:
            field_dict["columns_successfully_analyzed"] = columns_successfully_analyzed
        if total_collectors_failed is not UNSET:
            field_dict["total_collectors_failed"] = total_collectors_failed
        if total_collected_results is not UNSET:
            field_dict["total_collected_results"] = total_collected_results

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        executed_statistics_collectors = d.pop("executed_statistics_collectors", UNSET)

        columns_analyzed = d.pop("columns_analyzed", UNSET)

        columns_successfully_analyzed = d.pop("columns_successfully_analyzed", UNSET)

        total_collectors_failed = d.pop("total_collectors_failed", UNSET)

        total_collected_results = d.pop("total_collected_results", UNSET)

        collect_statistics_result = cls(
            executed_statistics_collectors=executed_statistics_collectors,
            columns_analyzed=columns_analyzed,
            columns_successfully_analyzed=columns_successfully_analyzed,
            total_collectors_failed=total_collectors_failed,
            total_collected_results=total_collected_results,
        )

        collect_statistics_result.additional_properties = d
        return collect_statistics_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AzureAuthenticationMode(str, Enum):
    CONNECTION_STRING = "connection_string"
    CREDENTIAL_CHAIN = "credential_chain"
    DEFAULT_CREDENTIALS = "default_credentials"
    SERVICE_PRINCIPAL = "service_principal"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.provider_type import ProviderType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.provider_sensor_definition_spec import ProviderSensorDefinitionSpec


T = TypeVar("T", bound="ProviderSensorModel")


@_attrs_define
class ProviderSensorModel:
    """Provider sensor model

    Attributes:
        provider_type (Union[Unset, ProviderType]):
        provider_sensor_definition_spec (Union[Unset, ProviderSensorDefinitionSpec]):
        sql_template (Union[Unset, str]): Provider specific Jinja2 SQL template
        error_sampling_template (Union[Unset, str]): Provider specific Jinja2 SQL template for capturing error samples
        custom (Union[Unset, bool]): Whether the provider sensor is a User Home provider sensor
        built_in (Union[Unset, bool]): This is a DQOps built-in provider sensor, whose parameters cannot be changed.
        can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can update or delete this object.
        yaml_parsing_error (Union[Unset, str]): Optional parsing error that was captured when parsing the YAML file.
            This field is null when the YAML file is valid. If an error was captured, this field returns the file parsing
            error message and the file location.
    """

    provider_type: Union[Unset, ProviderType] = UNSET
    provider_sensor_definition_spec: Union[Unset, "ProviderSensorDefinitionSpec"] = (
        UNSET
    )
    sql_template: Union[Unset, str] = UNSET
    error_sampling_template: Union[Unset, str] = UNSET
    custom: Union[Unset, bool] = UNSET
    built_in: Union[Unset, bool] = UNSET
    can_edit: Union[Unset, bool] = UNSET
    yaml_parsing_error: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        provider_type: Union[Unset, str] = UNSET
        if not isinstance(self.provider_type, Unset):
            provider_type = self.provider_type.value

        provider_sensor_definition_spec: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.provider_sensor_definition_spec, Unset):
            provider_sensor_definition_spec = (
                self.provider_sensor_definition_spec.to_dict()
            )

        sql_template = self.sql_template
        error_sampling_template = self.error_sampling_template
        custom = self.custom
        built_in = self.built_in
        can_edit = self.can_edit
        yaml_parsing_error = self.yaml_parsing_error

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if provider_type is not UNSET:
            field_dict["providerType"] = provider_type
        if provider_sensor_definition_spec is not UNSET:
            field_dict["providerSensorDefinitionSpec"] = provider_sensor_definition_spec
        if sql_template is not UNSET:
            field_dict["sqlTemplate"] = sql_template
        if error_sampling_template is not UNSET:
            field_dict["errorSamplingTemplate"] = error_sampling_template
        if custom is not UNSET:
            field_dict["custom"] = custom
        if built_in is not UNSET:
            field_dict["builtIn"] = built_in
        if can_edit is not UNSET:
            field_dict["canEdit"] = can_edit
        if yaml_parsing_error is not UNSET:
            field_dict["yamlParsingError"] = yaml_parsing_error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.provider_sensor_definition_spec import (
            ProviderSensorDefinitionSpec,
        )

        d = src_dict.copy()
        _provider_type = d.pop("providerType", UNSET)
        provider_type: Union[Unset, ProviderType]
        if isinstance(_provider_type, Unset):
            provider_type = UNSET
        else:
            provider_type = ProviderType(_provider_type)

        _provider_sensor_definition_spec = d.pop("providerSensorDefinitionSpec", UNSET)
        provider_sensor_definition_spec: Union[Unset, ProviderSensorDefinitionSpec]
        if isinstance(_provider_sensor_definition_spec, Unset):
            provider_sensor_definition_spec = UNSET
        else:
            provider_sensor_definition_spec = ProviderSensorDefinitionSpec.from_dict(
                _provider_sensor_definition_spec
            )

        sql_template = d.pop("sqlTemplate", UNSET)

        error_sampling_template = d.pop("errorSamplingTemplate", UNSET)

        custom = d.pop("custom", UNSET)

        built_in = d.pop("builtIn", UNSET)

        can_edit = d.pop("canEdit", UNSET)

        yaml_parsing_error = d.pop("yamlParsingError", UNSET)

        provider_sensor_model = cls(
            provider_type=provider_type,
            provider_sensor_definition_spec=provider_sensor_definition_spec,
            sql_template=sql_template,
            error_sampling_template=error_sampling_template,
            custom=custom,
            built_in=built_in,
            can_edit=can_edit,
            yaml_parsing_error=yaml_parsing_error,
        )

        provider_sensor_model.additional_properties = d
        return provider_sensor_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
